<?php
/*
   ----------------------------------------------------------------------
   GLPI - Gestionnaire Libre de Parc Informatique
   Copyright (C) 2003-2008 by the INDEPNET Development Team.

   http://indepnet.net/   http://glpi-project.org/
   ----------------------------------------------------------------------

   LICENSE

   This file is part of GLPI.

   GLPI is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   GLPI is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with GLPI; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   ------------------------------------------------------------------------
 */

// ----------------------------------------------------------------------
// Original Author of file: Remi Collet
// Purpose of file: Classes for XML-RPC plugin
// ----------------------------------------------------------------------

if (!defined('GLPI_ROOT')){
	die("Sorry. You can't access directly to this file");
	}

class WebServices extends CommonDBTM {

	function __construct () {
		$this->table="glpi_plugin_webservices";
		$this->type=PLUGIN_WEBSERVICES_TYPE;
		$this->entity_assign=true;
		$this->may_be_recursive=true;
		$this->dohistory=true;
	}

	function defineTabs($ID,$withtemplate){
		global $LANG;

		$ong=array();

		$ong[1]  = $LANG["title"][26];

		if ($ID>0) {
			$ong[2]  = $LANG["webservices"][7];
			$ong[12] = $LANG["title"][38];
		}

		return $ong;
	}

	function prepareInputForAdd($input) {
		return $this->prepareInputForUpdate($input);
	}

	function prepareInputForUpdate($input) {

		if (isset($input['username'])){
			if (empty($input['username'])) {
				$input['username']="NULL";
				$input['password']="NULL";
			} else {
				$input['password']=md5(isset($input['_password']) ? $input['_password'] : '');
			}
		}

		if (isset($input['_start']) && isset($input['_end'])){
			if (empty($input['_start'])) {
				$input['ip_start']="NULL";
				$input['ip_end']="NULL";
			} else {
				$input['ip_start']=ip2long($input['_start']);
				if (empty($input['_end'])) {
					$input['ip_end']=$input['ip_start'];
				} else {
					$input['ip_end']=ip2long($input['_end']);
				}
				if ($input['ip_end'] < $input['ip_start']) {
					$tmp = $input['ip_end'];
					$input['ip_end'] = $input['ip_start'];
					$input['ip_start'] = $tmp;
				}
			}

		}
		return $input;
	}

	function showForm ($target,$ID,$withtemplate='') {

		GLOBAL $CFG_GLPI, $LANG;

		GLOBAL $DB;

		if (!haveRight("config", "r")) {
			echo "ERROR $ID";
			return false;
		}

		if ($ID>0){
			if (!$this->can($ID,'r')) return false;
		} else {
			if (!$this->can(-1,'w')) return false;
			$this->getEmpty();
		}

		$this->showTabs($ID, $withtemplate,$_SESSION['glpi_tab']);

		$canedit=$this->can($ID,'w');
		$canrecu=$this->can($ID,'recursive');

		echo "<form method='post' name=form action=\"$target\">";
		if (empty($ID)||$ID<0){
				echo "<input type='hidden' name='FK_entities' value='".$_SESSION["glpiactive_entity"]."'>";
			}
		echo "<div class='center' id='tabsbody'>";
		echo "<table class='tab_cadre_fixe' >";

		$this->showFormHeader($ID);

		echo "<tr class='tab_bg_1'><td>".$LANG["common"][16].":	</td><td>";
		autocompletionTextField("name",$this->table,"name",$this->fields["name"],40,$this->fields["FK_entities"]);
		echo "</td></tr>";

		echo "<tr class='tab_bg_1'><td >".$LANG["webservices"][1].":</td><td>";
		dropdownYesNo("active",$this->fields["active"]);
		echo "</td></tr>";

      echo "<tr class='tab_bg_1'><td >".$LANG["webservices"][9].":</td><td>";
      dropdownYesNo("deflate",$this->fields["deflate"]);
      echo "  " . nl2br($LANG["webservices"][14]) . "</td></tr>";

		echo "<tr class='tab_bg_1'><td >".$LANG["webservices"][6].":</td><td>";
		dropdownArrayValues("do_log",
         array(0=>$LANG['choice'][0], // Non
            1=>$LANG['title'][38], // Historique
            2=>$LANG['Menu'][30]), // Jounaux
         $this->fields["do_log"]);
		echo "</td></tr>";

      echo "<tr class='tab_bg_1'><td >".$LANG['setup'][137].":</td><td>";
      dropdownYesNo("debug",$this->fields["debug"]);
      echo "</td></tr>";

		echo "<tr class='tab_bg_1'><td >".$LANG["webservices"][2].":</td><td>";
		echo "<input type='text' name='pattern' value='".$this->fields["pattern"]."'  size='40'>";
		echo "</td></tr>";

		echo "<tr class='tab_bg_1'><td >".$LANG["webservices"][3].":</td><td>";
		echo "<input type='text' name='_start' value='".
			($this->fields["ip_start"] ? long2ip($this->fields["ip_start"]) : '') .
			"'  size='18'> - ";
		echo "<input type='text' name='_end' value='" .
			($this->fields["ip_end"] ? long2ip($this->fields["ip_end"]) : '') .
			"'  size='18'></td></tr>";

		echo "<tr class='tab_bg_1'><td >".$LANG["webservices"][4].":</td><td>";
		echo "<input type='text' name='username' value='".$this->fields["username"]."'  size='40'>";
		echo "</td></tr>";

		echo "<tr class='tab_bg_1'><td >".$LANG["webservices"][5].":</td><td>";
		echo "<input type='password' name='_password' value=''  size='40'>";
		echo "</td></tr>";

		echo "<tr class='tab_bg_1'><td valign='top'>".$LANG["common"][25].
			":</td><td valign='top'><textarea  cols='60' rows='4' name='comments' >".$this->fields["comments"]."</textarea></td>";

		if ($canedit) {

			if (empty($ID)||$ID<0){
				echo "<tr>";
				echo "<td class='tab_bg_2' valign='top' colspan='2'>";
				echo "<div align='center'><input type='submit' name='add' value=\"".$LANG["buttons"][8]."\" class='submit'>";
				echo "</div></td></tr>";

			} else {

				echo "<tr>";
				echo "<td class='tab_bg_2'  colspan='2' valign='top'><div align='center'>";
				echo "<input type='hidden' name='ID' value=\"$ID\">\n";
				echo "<input type='submit' name='update' value=\"".$LANG["buttons"][7]."\" class='submit' >";
				if ($ID>1) {
					// 1st rule must not be deleted (used for access denied logging)
					echo "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<input type='submit' name='delete' value=\"".$LANG["buttons"][6]."\" class='submit'>";
				}
				echo "</div></td></tr>";
			}
		}
		echo "</table></div></form>";
		echo "<div id='tabcontent'></div>";
		echo "<script type='text/javascript'>loadDefaultTab();</script>";

		return true;
	}

	function showMethods() {
		global $LANG, $WEBSERVICES_METHOD;

		echo "<div class='center'><br><table class='tab_cadrehov'>";
		echo "<tr><th colspan='4'>".$LANG["webservices"][8]."</th></tr>";
		echo "<tr><th>".$LANG["webservices"][10]."</th>" .
			"<th>".$LANG["webservices"][12]."</th>" .
			"<th>".$LANG["webservices"][11]."</th>" .
			"<th>".$LANG["webservices"][13]."</th></tr>";

		// Allow all plugins to register their methods
		$WEBSERVICES_METHOD=array();
		doHook("webservices");

		foreach ($WEBSERVICES_METHOD as $method => $function) {
			// if (preg_match("/".$this->fields['pattern']."/",$method)) { // Could be different from MySQL REGEXP

			// Display if MySQL REGEXP match
			if (countElementsInTable($this->table,
				"ID='".$this->fields['ID']."' AND '".addslashes($method)."' REGEXP pattern")>0) {
				if (preg_match('/^plugin_(.*)_method/', $function, $res)) {
					$plugin=$res[1];
				} else {
					$plugin="&nbsp;";
				}
				$call=(function_exists($function) ? $LANG["choice"][1] : $LANG["choice"][0]);
				$color=(function_exists($function) ? "greenbutton" : "redbutton");
				echo "<tr class='tab_bg_1'><td><strong>$method</strong></td><td>$plugin</td><td>$function</td>" .
						"<td><img src=\"".GLPI_ROOT."/pics/$color.png\" alt='ok'>&nbsp;$call</td></tr>";
			}
		}

		echo "</table></div>";
	}
}


class SoapWebService {
   function genericExecute($params) {
      $method = $params['method'];
      unset($params['method']);
      return plugin_webservice_execute($method,$params,WEBSERVICE_PROTOCOL_SOAP);
   }
}
?>