<?php

/*
   ----------------------------------------------------------------------
   GLPI - Gestionnaire Libre de Parc Informatique
   Copyright (C) 2003-2008 by the INDEPNET Development Team.

   http://indepnet.net/   http://glpi-project.org/
   ----------------------------------------------------------------------

   LICENSE

   This file is part of GLPI.

   GLPI is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   GLPI is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with GLPI; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   ------------------------------------------------------------------------
 */

// ----------------------------------------------------------------------
// Original Author of file: Remi Collet
// Purpose of file: Classes for XML-RPC plugin
// ----------------------------------------------------------------------

// Standard error Code
define('WEBSERVICES_ERROR_NOTFOUND', 10);
define('WEBSERVICES_ERROR_MISSINGPARAMETER', 11);
define('WEBSERVICES_ERROR_LOGINFAILED', 12);
define('WEBSERVICES_ERROR_NOTAUTHENTICATED', 13);
define('WEBSERVICES_ERROR_BADPARAMETER', 14);
define('WEBSERVICES_ERROR_FAILED', 15);
define('WEBSERVICES_ERROR_NOTALLOWED', 16);

define('WEBSERVICES_REGEX_DATETIME', '/^(19|20)\d{2}-(0|1)\d-[0-3]\d [0-2]\d:[0-5]\d:[0-5]\d$/');
define('WEBSERVICES_REGEX_DATE', '/^(19|20)\d{2}-(0|1)\d-[0-3]\d$/');

//Communication protocols
define('WEBSERVICE_PROTOCOL_XMLRPC', 'xml-rpc');
define('WEBSERVICE_PROTOCOL_SOAP', 'soap');
define('LOGFILENAME', 'webservices');

if (!defined('GLPI_ROOT')) {
	die("Sorry. You can't access directly to this file");
}

/**
 * Build a XML-RPC Response with an error status
 *
 * @param protocol the communication protocol used
 * @param $code interger : value of the error code
 * @param $code string : desciption of the error
 *
 * @return an response (fault) ready to be encode
 */
function plugin_webservices_Error($protocol, $code, $message = '', $more = '') {

	if (empty ($message))
		switch ($code) {
			case WEBSERVICES_ERROR_NOTFOUND :
				$message = 'Not found';
				break;
			case WEBSERVICES_ERROR_MISSINGPARAMETER :
				$message = 'Missing parameter';
				break;
			case WEBSERVICES_ERROR_LOGINFAILED :
				$message = 'Login failed';
				break;
			case WEBSERVICES_ERROR_NOTAUTHENTICATED :
				$message = 'Not authenticated';
				break;
			case WEBSERVICES_ERROR_BADPARAMETER :
				$message = 'Bad parameter';
				break;
			case WEBSERVICES_ERROR_FAILED :
				$message = 'Command have failed';
				break;
			case WEBSERVICES_ERROR_NOTALLOWED :
				$message = 'Command not allowed';
				break;
			default :
				$message = 'Error';
		}
	if (!empty ($more)) {
		$message .= ' (' . $more . ')';
	}

	switch ($protocol) {
		default :
			return array (
				"faultCode" => intval($code),
				"faultString" => $message
			);
		case WEBSERVICE_PROTOCOL_SOAP :
			return new SoapFault($message, intval($code));
	}
}

function plugin_webservice_isError($protocol, $response) {
	switch ($protocol) {
		case WEBSERVICE_PROTOCOL_SOAP :
			return is_a($response, 'SoapFault');
		default :
			return (isset ($response['faultCode']));
	}
}


function plugin_webservices_getRequestTypeName($requestTypeId) {
   global $LANG;

         switch ($requestTypeId) {
            case 1 :
               $nom=$LANG['Menu'][31];
               break;

            case 2 :
               $nom=$LANG['setup'][14];
               break;

            case 3 :
               $nom=$LANG['help'][35];
               break;

            case 4 :
               $nom=$LANG['tracking'][34];
               break;

            case 5 :
               $nom=$LANG['tracking'][35];
               break;

            case 6 :
               $nom=$LANG['common'][62];
               break;

            default :
               $nom='';
         }
         return $nom;
}



/**
 * The simplest method
 *
 * @param $params array of option : ignored
 *
 * @return an response ready to be encode
 */
function plugin_webservices_method_test($params, $protocol) {
	global $PLUGIN_HOOKS;

	if (isset ($params['help'])) {
		return array (
			'help' => 'bool,optional'
		);
	}

	$resp = array (
		'glpi' => GLPI_VERSION
	);

	$plugin = new Plugin();
	foreach ($PLUGIN_HOOKS['webservices'] as $name => $fct) {
		if ($plugin->getFromDBbyDir($name)) {
			$resp[$name] = $plugin->fields['version'];
		}
	}

	return $resp;
}

/**
 * This method return the list of all method
 *
 * TODO: to be filter for the client rights
 *
 * @param $params array of option : ignored
 *
 * @return an response ready to be encode
 */
function plugin_webservices_method_list($params, $protocol) {
	global $WEBSERVICES_METHOD;

	if (isset ($params['help'])) {
		return array (
			'help' => 'bool,optional'
		);
	}

	return $WEBSERVICES_METHOD;
}

/**
 * This method return the entities list for the client
 *
 * @param $params array of option : ignored
 *
 * @return an response ready to be encode (ID + completename)
 */
function plugin_webservices_method_listEntities($params, $protocol) {

	global $DB, $LANG, $CFG_GLPI;

	if (isset ($params['help'])) {
		return array (
			'count' => 'bool,optional',
			'start' => 'integer,optional',
			'limit' => 'integer,optional',
			'help' => 'bool,optional'
		);
	}

	// Should never occurs, just to show howto to handle an error
	if (!count($_SESSION['glpiactiveentities'])) {
		return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_NOTFOUND);
	}

	$resp = array ();

	// Count only
	if (isset ($params['count'])) {
		$resp['count'] = count($_SESSION['glpiactiveentities']);
		return $resp;
	}
	$start = 0;
	$limit = $CFG_GLPI["list_limit_max"];
	if (isset ($params['limit']) && is_numeric($params['limit'])) {
		$limit = $params['limit'];
	}
	if (isset ($params['start']) && is_numeric($params['start'])) {
		$start = $params['start'];
	}

	// Root entity
	if (isset ($_SESSION['glpiactiveentities'][0]) && !$start) {
		$resp[] = array (
			'ID' => 0,
			'completename' => $LANG["entity"][2]
		);
		$limit--;
	}

	// Other allowed entities
	foreach ($DB->request("glpi_entities", array (
			'ID' => $_SESSION['glpiactiveentities'],
			'ORDER' => 'completename',
			'START' => $start,
			'LIMIT' => $limit
		)) as $entity) {
		$resp[] = array (
			'id' => $entity['ID'],
			'completename' => $entity['completename']
		);
	}

	return $resp;
}

/**
 * This method return GLPI status (same as status.php)
 *
 * @param $params array of option : ignored
 *
 * @return an response ready to be encode
 */
function plugin_webservices_method_status($params) {
	global $DB;

	if (isset ($params['help'])) {
		return array (
			'help' => 'bool,optional'
		);
	}

	$resp = array ();

	$ok_master = true;
	$ok_slave = true;
	$ok = true;

	// Check slave server connection
	if (isDBSlaveActive()) {
		if (establishDBConnection(true, true, false)) {
			$resp['slavedb'] = "ok";
		} else {
			$resp['slavedb'] = "offline";
			$ok_slave = false;
		}
	} else {
		$resp['slavedb'] = "not configured";
	}

	// Check main server connection
	if (establishDBConnection(false, true, false)) {
		$resp['maindb'] = "ok";
	} else {
		$resp['slavedb'] = "offline";
		$ok_master = false;
	}
	// Slave and master ok;
	$ok = $ok_slave && $ok_master;

	// Check session dir (usefull when NFS mounted))
	if (is_dir(GLPI_SESSION_DIR) && is_writable(GLPI_SESSION_DIR)) {
		$resp['sessiondir'] = "ok";
	} else {
		$resp['sessiondir'] = "not writable";
		$ok = false;
	}
	// Reestablished DB connection
	if (($ok_master || $ok_slave) && establishDBConnection(false, false, false)) {

		// Check OCS connections
		$query = "SELECT ID, name FROM glpi_ocs_config";
		if ($result = $DB->query($query)) {
			if ($DB->numrows($result)) {
				while ($data = $DB->fetch_assoc($result)) {
					if (checkOCSconnection($data['ID'])) {
						$resp['OCS_' . $data['name']] = "ok";
					} else {
						$resp['OCS_' . $data['name']] = "offline";
						$ok = false;
					}
				}
			}
		}

		// Check Auth connections
		$auth = new Identification();
		$auth->getAuthMethods();
		$ldap_methods = $auth->auth_methods["ldap"];

		if (count($ldap_methods)) {
			foreach ($ldap_methods as $method) {
				if (try_connect_ldap($method['ldap_host'], $method['ldap_port'], $method["ldap_rootdn"], $method["ldap_pass"], $method["ldap_use_tls"], "", "", $method["ldap_opt_deref"], $method['ID'])) {
					$resp['LDAP_' . $method['name']] = "ok";
				} else {
					$resp['LDAP_' . $method['name']] = "offline";
					$ok = false;
				}
			}
		}
	}

	if ($ok) {
		$resp['glpi'] = "ok";
	} else {
		$resp['glpi'] = "error";
	}
	return $resp;
}

/**
 * This method return  groups list allowed
 * for a authenticated user
 *
 * @param $params array of options
 *
 * @return an response ready to be encode (ID + completename)
 */
function plugin_webservices_method_listGroups($params, $protocol) {

	global $DB, $LANG, $CFG_GLPI;

	if (isset ($params['help'])) {
		return array (
			'count' => 'bool,optional',
			'start' => 'integer,optional',
			'limit' => 'integer,optional',
			'help' => 'bool,optional'
		);
	}

	if (!isset ($_SESSION['glpiID'])) {
		return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_NOTAUTHENTICATED);
	}

	$resp = array ();
	if (isset ($params['count'])) {
		$resp['count'] = countElementsInTable('glpi_groups', getEntitiesRestrictRequest('', 'glpi_groups'));
		return $resp;
	}
	$start = 0;
	$limit = $CFG_GLPI["list_limit_max"];
	if (isset ($params['limit']) && is_numeric($params['limit'])) {
		$limit = $params['limit'];
	}
	if (isset ($params['start']) && is_numeric($params['start'])) {
		$start = $params['start'];
	}
	foreach ($DB->request('glpi_groups', array (
			'FK_entities' => $_SESSION['glpiactiveentities'],
			'ORDER' => 'ID',
			'START' => $start,
			'LIMIT' => $limit
		)) as $data) {
		$resp[$data['ID']] = array (
			'id' => $data['ID'],
			'entities_id' => $data['FK_entities'],
			'is_recursive' => $data['recursive'],
			'name' => $data['name'],
			'comment' => $data['comments'],
			'users_id' => $data['FK_users'],
			'ldap_field' => $data['ldap_field'],
			'ldap_value' => $data['ldap_value'],
			'ldap_group_dn' => $data['ldap_group_dn'],
			'member' => (in_array($data['ID'], $_SESSION['glpigroups']) ? 1 : 0)
		);
	}
	return $resp;
}

/**
 * Change the current entity(ies) of a authenticated user
 *
 * @param $params array of options
 * @return array of hashtable
 */
function plugin_webservices_method_listHelpdeskTypes($params, $protocol) {

	global $DB, $LANG, $CFG_GLPI;

	if (isset ($params['help'])) {
		return array (
			'help' => 'bool,optional'
		);
	}

	if (!isset ($_SESSION['glpiID'])) {
		return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_NOTAUTHENTICATED);
	}
	if (!isset ($_SESSION["glpiactiveprofile"]["helpdesk_hardware"]) || !$_SESSION["glpiactiveprofile"]["helpdesk_hardware"]) {
		// No right to attach a item to a tickeet
		return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_NOTFOUND);
	}

	$resp[] = array (
		'id' => 0,
		'name' => $LANG['help'][30]
	); // General

	if ($_SESSION["glpiactiveprofile"]["helpdesk_hardware"] & pow(2, HELPDESK_MY_HARDWARE)) {
		$resp[] = array (
			'id' => -1,
			'name' => $LANG['tracking'][1]
		);
	}
	if ($_SESSION["glpiactiveprofile"]["helpdesk_hardware"] & pow(2, HELPDESK_ALL_HARDWARE)) {
		$types = getAllTypesForHelpdesk();

		foreach ($types as $id => $name) {
			$resp[] = array (
				'id' => $id,
				'name' => $name
			);
		}
	}
	return $resp;
}

/**
 * List the Item for a Helpesk
 * for a authenticated user
 *
 * @param $params array of options (itemtype, name, group, itemsubtype, id2name)
 * @return array of hashtable
 */
function plugin_webservices_method_listHelpdeskItems($params, $protocol) {

	global $DB, $LANG, $CFG_GLPI, $LINK_ID_TABLE;

	if (isset ($params['help'])) {
		return array (
			'itemtype' => 'integer,mandatory',
			'count' => 'bool,optional',
			'start' => 'integer,optional',
			'limit' => 'integer,optional',
			'group' => 'integer,optional',
			'name' => 'string,optional',
			'state' => 'integer,optional',
			'itemsubtype' => 'integer,optional',
			'id2name' => 'bool,optional',
         'entity' => 'integer,optional',
			'help' => 'bool,optional'
		);
	}

	if (!isset ($_SESSION['glpiID'])) {
		return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_NOTAUTHENTICATED);
	}
	if (!isset ($_SESSION["glpiactiveprofile"]["helpdesk_hardware"]) || !$_SESSION["glpiactiveprofile"]["helpdesk_hardware"]) {
		// No right to attach a item to a tickeet
		return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_NOTALLOWEDl);
	}

	if (!isset ($params['itemtype'])) {
		return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_MISSINGPARAMETER);
	}
	if (!is_numeric($params['itemtype'])) {
		return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_BADPARAMETER);
	}

	if (!isset ($_SESSION["glpiactiveprofile"]["helpdesk_hardware"]) || !$_SESSION["glpiactiveprofile"]["helpdesk_hardware"]) {
		// No right to attach a item to a tickeet
		return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_NOTALLOWED, $protocol);
	}

	$start = 0;
	if (isset ($params['start']) && is_numeric($params['start'])) {
		$start = $params['start'];
	}
	$limit = $CFG_GLPI["list_limit_max"];
	if (isset ($params['limit']) && is_numeric($params['limit'])) {
		$limit = $params['limit'];
	}

   $resp = array ();
   if (isset ($params['count'])) {
      $resp['count'] = 0;
   }

	// Partial code from dropdownMyDevices
	if ($params['itemtype'] < 0 && $_SESSION["glpiactiveprofile"]["helpdesk_hardware"] & pow(2, HELPDESK_MY_HARDWARE)) {
		// My items
		foreach ($CFG_GLPI["linkuser_types"] as $type) {
			if ($limit > 0 && isPossibleToAssignType($type)) {
				$where = " deleted=0 ";
				if (count($_SESSION['glpigroups']) && haveRight("show_group_hardware", "1") && in_array($type, $CFG_GLPI["linkgroup_types"])) {
					$where .= " AND (FK_users='" . $_SESSION['glpiID'] . "' OR FK_groups IN(" . implode(',', $_SESSION['glpigroups']) . "))";
				} else {
					$where .= " AND FK_users='" . $_SESSION['glpiID'] . "'";
				}
				if (in_array($LINK_ID_TABLE[$type], $CFG_GLPI["template_tables"])) {
					$where .= " AND is_template='0' ";
				}
				$where .= getEntitiesRestrictRequest(" AND", $LINK_ID_TABLE[$type], '', '', isset ($CFG_GLPI['recursive_type'][$type]));

				$nb = countElementsInTable($LINK_ID_TABLE[$type], $where);
				if (isset ($params['count'])) {
					// Only count
					$resp['count'] += $nb;
				} else if ($start >= $nb) {
					// Skip this type
					$start -= $nb;
				} else {
					$query = "SELECT * FROM " . $LINK_ID_TABLE[$type] .
					" WHERE $where ORDER BY ID LIMIT $start,$limit";
					foreach ($DB->request($query) as $data) {
						$out = array ();
						$out['itemtype'] = $type;
						$out['id'] = $data['ID'];
						$out['name'] = $data["name"];
						$output = $data["name"];

						$from = array (
							'serial',
							'otherserial',
							'FK_users',
							'FK_groups',
							'type',
							'state'
						);
						$dest = array (
							'serial',
							'otherserial',
							'users_id',
							'groups_id',
							'itemsubtype',
							'states_id'
						);
						foreach ($from as $num => $field) {
							if (isset ($data[$field])) {
								$out[$dest[$num]] = $data[$field];
							}
						}
						$resp[] = $out;

						// For next type
						if ($start)
							$start--;
						$limit--;
					}
				}
			} // allowed
		} // each type
	} // My items

   $type = $params['itemtype'];
   if ($type > 0 && $_SESSION["glpiactiveprofile"]["helpdesk_hardware"] & pow(2, HELPDESK_ALL_HARDWARE) && isPossibleToAssignType($type)) {
      // All items of a type

      // Entity
      if (isset ($params['entity'])) {
         if (!haveAccessToEntity($params['entity'])) {
            return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_NOTALLOWED, '', 'entity');
         }
         $ent = $params['entity'];
      } else {
         $ent = '';
      }
      $where = getEntitiesRestrictRequest('', $LINK_ID_TABLE[$type], '', $ent,
                                          isset ($CFG_GLPI['recursive_type'][$type]));

		if (in_array($LINK_ID_TABLE[$type], $CFG_GLPI["deleted_tables"])) {
			$where .= " AND deleted=0";
		}
		if (in_array($LINK_ID_TABLE[$type], $CFG_GLPI["template_tables"])) {
			$where .= " AND is_template=0";
		}
		if (in_array($type, $CFG_GLPI["helpdesk_visible_types"])) {
			$where .= " AND helpdesk_visible=1 ";
		}
		if (isset ($params['group']) && is_numeric($params['group']) && in_array($type, $CFG_GLPI["linkgroup_types"]) && !in_array($type, $CFG_GLPI["helpdesk_visible_types"])) {
			$where .= " AND FK_groups='" . $params['group'] . "'";
		}
		if (isset ($params['name'])) {
			$where .= " AND name LIKE '" . addslashes($params['name']) . "'";
		}
		// TODO : 0.80 type field have disctinct name for each table
		if (isset ($params['itemsubtype']) && is_numeric($params['itemsubtype'])) {
			$where .= " AND type='" . $params['itemsubtype'] . "'";
		}
		if (isset ($params['state']) && is_numeric($params['state']) && in_array($type, $CFG_GLPI["state_types"])) {
			$where .= " AND state='" . $params['state'] . "'";
		}

		if (isset ($params['count'])) {
			$resp['count'] = countElementsInTable($LINK_ID_TABLE[$type], $where);
		} else {
			$query = "SELECT * FROM " . $LINK_ID_TABLE[$type] .
			" WHERE $where ORDER BY ID LIMIT $start,$limit";
			foreach ($DB->request($query) as $data) {
				$out = array ();
				$out['itemtype'] = $type;
				$out['id'] = $data['ID'];
				$out['name'] = $data["name"];
				$output = $data["name"];

				// TODO : 0.80 type field have disctinct name for each table
				$from = array (
					'serial',
					'otherserial',
					'FK_users',
					'FK_groups',
					'type',
					'state'
				);
				$dest = array (
					'serial',
					'otherserial',
					'users_id',
					'groups_id',
					'itemsubtype',
					'states_id'
				);
				foreach ($from as $num => $field) {
					if (isset ($data[$field])) {
						$out[$dest[$num]] = $data[$field];
					}
				}
				$resp[] = $out;
			}
		}
	} // All items

	if (isset ($params['id2name'])) {
		foreach ($resp as $k => $v) {
			if (isset ($resp[$k]['users_id'])) {
				$resp[$k]['users_name'] = html_clean(getUserName($resp[$k]['users_id']));
			}
			if (isset ($resp[$k]['groups_id'])) {
				$resp[$k]['groups_name'] = html_clean(getDropdownName('glpi_groups', $resp[$k]['groups_id']));
			}
		}
	}

	return $resp;
}

/**
 * List the Users of the entity, with search criterias
 * for a authenticated user
 *
 * @param $params array of options (user, group, location, login, name)
 * @return array of hashtable
 */
function plugin_webservices_method_listUsers($params, $protocol) {
	global $DB, $LANG, $CFG_GLPI;

	if (isset ($params['help'])) {
		return array (
			'count' => 'bool,optional',
			'start' => 'integer,optional',
			'limit' => 'integer,optional',
			'user' => 'integer,optional',
			'group' => 'integer,optional',
			'location' => 'integer,optional',
			'login' => 'string,optional',
			'name' => 'string,optional',
			'help' => 'bool,optional'
		);
	}

	if (!isset ($_SESSION['glpiID'])) {
		return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_NOTAUTHENTICATED);
	}

	$query = "LEFT JOIN glpi_users_profiles ON (glpi_users.ID = glpi_users_profiles.FK_users)" .
	getEntitiesRestrictRequest(" WHERE ", "glpi_users_profiles", '', $_SESSION['glpiactiveentities'], 1);

	if (isset ($params['user']) && is_numeric($params['user'])) {
		$query .= " AND glpi_users.ID='" . $params['user'] . "'";
	}
	if (isset ($params['group']) && is_numeric($params['group'])) {
		$query .= " AND glpi_users.ID IN (SELECT FK_users FROM glpi_users_groups WHERE FK_groups='" . $params['group'] . "')";
	}
	if (isset ($params['location']) && is_numeric($params['location'])) {
		$query .= " AND glpi_users.location='" . $params['location'] . "'";
	}
	if (isset ($params['login'])) {
		$query .= " AND glpi_users.name LIKE '" . addslashes($params['login']) . "'";
	}
	if (isset ($params['name'])) {
		if ($CFG_GLPI["name_display_order"] == FIRSTNAME_BEFORE) {
			$query .= " AND CONCAT(glpi_users.firstname,' ',glpi_users.realname) LIKE '" . addslashes($params['name']) . "'";
		} else {
			$query .= " AND CONCAT(glpi_users.realname,' ',glpi_users.firstname) LIKE '" . addslashes($params['name']) . "'";
		}
	}
	$resp = array ();
	if (isset ($params['count'])) {
		$query = "SELECT COUNT(DISTINCT glpi_users.ID) as count FROM glpi_users " . $query;
	} else {

		$start = 0;
		$limit = $CFG_GLPI["list_limit_max"];
		if (isset ($params['limit']) && is_numeric($params['limit'])) {
			$limit = $params['limit'];
		}
		if (isset ($params['start']) && is_numeric($params['start'])) {
			$start = $params['start'];
		}

		$query = "SELECT DISTINCT glpi_users.ID AS id, glpi_users.name, firstname, realname, email, phone,
		               glpi_dropdown_locations.completename AS locations_name
		         FROM glpi_users
		         LEFT JOIN glpi_dropdown_locations ON (glpi_users.location=glpi_dropdown_locations.ID)
		         " . $query . "
		         ORDER BY glpi_users.ID LIMIT $start,$limit";
	}

	foreach ($DB->request($query) as $data) {
		if (empty ($data['firstname']) && empty ($data['realname'])) {
			$data['displayname'] = $data['name'];
		} else
			if ($CFG_GLPI["name_display_order"] == FIRSTNAME_BEFORE) {
				$data['displayname'] = $data['firstname'] . ' ' . $data['realname'];
			} else {
				$data['displayname'] = $data['realname'] . ' ' . $data['firstname'];
			}
		$resp[] = $data;
	}
	return $resp;
}

/**
 * List value for a dropdown, with search criterias
 * for a authenticated user
 *
 * @param $params array of options (dropdown, id, parent, name)
 * @return array of hashtable
 */
function plugin_webservices_method_listDropdownValues($params, $protocol) {
	global $DB, $LANG, $CFG_GLPI;

	if (isset ($params['help'])) {
		return array (
			'dropdown' => 'string,mandatory',
			'id' => 'integer,optional',
			'parent' => 'integer,optional',
			'name' => 'string,optional',
			'help' => 'bool,optional'
		);
	}

	// TODO can't work in 0.80, must be rewrite (no dropdwn table, no common schema))'

	// Awfull hack: dropdown is entity assign, but object can be recursive...
	$entityhack = array (
		'glpi_dropdown_plugin_applicatifs_type'
	);

	if (!isset ($_SESSION['glpiID'])) {
		return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_NOTAUTHENTICATED);
	}

	if (!isset ($params['dropdown'])) {
		return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_MISSINGPARAMETER);
	}
	$table = 'glpi_dropdown_' . $params['dropdown'];

	if (!TableExists($table)) {
		return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_BADPARAMETER, '', $table);
	}
	$query = "SELECT * FROM `$table`";

	if (isset ($params['id']) && is_numeric($params['id'])) {
		$query .= " WHERE ID='" . $params['id'] . "'";
	} else {
		$query .= " WHERE 1";
	}
	if (in_array($table, $CFG_GLPI["specif_entities_tables"]) && !in_array($table, $entityhack)) {
		$query .= getEntitiesRestrictRequest(" AND ", $table);
	}
	if (isset ($params['parent']) && is_numeric($params['parent']) && in_array($table, $CFG_GLPI["dropdowntree_tables"])) {
		$query .= " AND parentID='" . $params['parent'] . "'";
	}
	if (isset ($params['name'])) {
		$query .= " AND completename LIKE '" . addslashes($params['name']) . "'";
	}
	$resp = array ();
	foreach ($DB->request($query) as $data) {
		$data['id'] = $data['ID'];
		unset ($data['ID']);

		if (isset ($data['parentID'])) {
			$data[$params['dropdown'] . "_id"] = $data['parentID'];
			unset ($data['parentID']);
		}
		if (isset ($data['FK_entities'])) {
			$data["entities_id"] = $data['FK_entities'];
			unset ($data['FK_entities']);
		}
		if (isset ($data['comments'])) {
			$data["comment"] = $data['comments'];
			unset ($data['comments']);
		}
		$resp[] = $data;
	}

	return $resp;
}

/**
 * Create a new ticket
 * for a authenticated user
 *
 * @param $params array of options
 *    (entity, user, group, date, itemtype, itemid, title, content, urgence, category)
 * @return array of hashtable
 */
function plugin_webservices_method_createTicket($params, $protocol) {
	global $CFG_GLPI, $INFOFORM_PAGES;

	if (isset ($params['help'])) {
		return array (
			'content' => 'string,mandatory',
			'title' => 'string,optional',
			'entity' => 'integer,optional',
			'urgence' => 'integer,optional',
			'category' => 'integer,optional',
			'user' => 'integer,optional',
			'group' => 'integer,optional',
			'date' => 'datetime,optional',
			'category' => 'integer,optional',
			'itemtype' => 'integer,optional',
			'item' => 'integer,optional',
			'help' => 'bool,optional'
		);
	}

	if (!isset ($_SESSION['glpiID'])) {
		return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_NOTAUTHENTICATED);
	}

	// Mandatory fields by config
	if (// ignore config : always mandatory $CFG_GLPI["ticket_content_mandatory"] &&
	 (!isset ($params['content']) || empty ($params['content']))) {
		return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_MISSINGPARAMETER, '', 'content');
	}
	if ($CFG_GLPI["ticket_title_mandatory"] && (!isset ($params['title']) || empty ($params['title']))) {
		return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_MISSINGPARAMETER, '', 'title');
	}
	if ($CFG_GLPI["ticket_category_mandatory"] && (!isset ($params['category']) || !is_numeric($params['category']))) {
		return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_MISSINGPARAMETER, '', 'category');
	}

   // ===== Build the Ticket =====
   // author : always the logged user
   $data = array (
      'author' => $_SESSION['glpiID'], // Requester / Victime
      'recipient' => $_SESSION['glpiID'], // Recorder
      'request_type' => 6, // source = other
      'status' => 'new',
      'contents' => addslashes(clean_cross_side_scripting_deep($params["content"])),
      'device_type' => 0,
      'computer' => 0
   );

	// Title : optional (default = start of contents set by add method)
	if (isset ($params["title"])) {
		$data['name'] = addslashes(clean_cross_side_scripting_deep($params["title"]));
	}

	// entity : optionnal, default = current one
	if (!isset ($params['entity'])) {
		$data['FK_entities'] = $_SESSION['glpiactive_entity'];
	} else
		if (!is_numeric($params['entity']) || !in_array($params['entity'], $_SESSION['glpiactiveentities'])) {
			return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_BADPARAMETER, '', 'entity');
		} else {
			$data['FK_entities'] = $params['entity'];
		}

	// user (author) : optionnal,  default = current one
	if (isset ($params['user'])) {
		if (!is_numeric($params['user'])) {
			return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_BADPARAMETER, '', 'user');
		} else {
			$data['author'] = $params['user'];
		}
   }

	// group (author) : optionnal,  default = none
	if (!isset ($params['group'])) {
		$data['FK_group'] = 0;
	} else
		if (!is_numeric($params['group'])) {
			return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_BADPARAMETER, '', 'group');
		} else {
			$data['FK_group'] = $params['group'];
		}

	// date (open) : optional, default set by add method
	if (isset ($params["date"])) {
		if (preg_match(WEBSERVICES_REGEX_DATETIME, $params["date"])) {
			$data['date'] = $params['date'];
		} else {
			return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_BADPARAMETER, '', 'date');
		}
	}

   if (isset ($params["itemtype"]) && !$params["itemtype"]) {
      unset($params["itemtype"]);
   }
   if (isset ($params["item"]) && !$params["item"]) {
      unset($params["item"]);
   }
	// Item type + id
	if (isset ($params["itemtype"])) {
		if (!isset ($params["item"])) {
			return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_MISSINGPARAMETER, '', 'item');
		}
		if (!is_numeric($params["itemtype"]) || !isset ($INFOFORM_PAGES[$params["itemtype"]])) {
			return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_BADPARAMETER, '', 'itemtype=' . $params["itemtype"]);
		}
	}
	if (isset ($params["item"])) {
		if (!isset ($params["itemtype"])) {
			return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_MISSINGPARAMETER, '', 'itemtype');
		}
		if (!is_numeric($params["item"]) || $params["item"] <= 0) {
			return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_BADPARAMETER, '', 'item=' . $params["item"]);
		}

		// Both ok
		$data['device_type'] = $params['itemtype'];
		$data['computer'] = $params['item'];
	}

	// urgence (priority while not exists) : optionnal,  default = 3
	if (!isset ($params['urgence'])) {
		$data['priority'] = 3;
	} else
		if (!is_numeric($params['urgence']) || $params['urgence'] < 1 || $params['urgence'] > 5) {
			return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_BADPARAMETER, '', 'urgence');
		} else {
			$data['priority'] = $params['urgence'];
		}

	// category : optionnal
	if (isset ($params['category'])) {
		if (!is_numeric($params['category']) || $params['category'] < 1) {
			return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_BADPARAMETER, '', 'category');
		} else {
			$data['category'] = $params['category'];
		}
	}

   $ticket = new Job();
   if ($newID = $ticket->add($data)) {
      return plugin_webservices_method_getTicket(array (
            'ticket' => $newID), $protocol);
   }
   return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_FAILED, '', (isset ($_SESSION["MESSAGE_AFTER_REDIRECT"]) ? $_SESSION["MESSAGE_AFTER_REDIRECT"] : ''));
}

/**
 * Add a followup to a existing ticket
 * for a authenticated user
 *
 * @param $params array of options (ticket, content)
 * @return array of hashtable
 */
function plugin_webservices_method_addTicketFollowup($params, $protocol) {

	if (isset ($params['help'])) {
		return array (
			'ticket' => 'integer,mandatory',
			'content' => 'string,mandatory',
			'help' => 'bool,optional'
		);
	}
	if (!isset ($_SESSION['glpiID'])) {
		return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_NOTAUTHENTICATED);
	}
	$ticket = new Job();

	if (!isset ($params['ticket'])) {
		return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_MISSINGPARAMETER, '', 'ticket');

	} else
		if (!is_numeric($params['ticket'])) {
			return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_BADPARAMETER, '', 'ticket');

		} else
			if (!$ticket->getFromDB($params['ticket']) || !$ticket->canView()) {
				return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_NOTFOUND);
			}
	if (!$ticket->canAddFollowups()) {
		return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_NOTALLOWED);
	}

	if (!isset ($params['content'])) {
		return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_MISSINGPARAMETER, '', 'content');
	}

	$followup = new Followup();
	$data = array (
		'tracking' => $params['ticket'],
		'contents' => addslashes(clean_cross_side_scripting_deep($params["content"]))
	);
	if ($followup->add($data)) {
		return plugin_webservices_method_getTicket(array (
			'ticket' => $params['ticket']), $protocol);
	}
	return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_FAILED, '', (isset ($_SESSION["MESSAGE_AFTER_REDIRECT"]) ? $_SESSION["MESSAGE_AFTER_REDIRECT"] : ''));
}

/**
 * Add a document to a existing ticket
 * for a authenticated user
 *
 * @param $params array of options (ticket, uri, name, base64, comment)
 *        only one of uri and base64 must be set
 *        name is mandatory when base64 set, for extension check (filename)
 *
 * @return array of hashtable
 */
function plugin_webservices_method_addTicketDocument($params, $protocol) {
	global $LANG, $CFG_GLPI;

	if (isset ($params['help'])) {
		return array (
			'ticket' => 'integer,mandatory',
			'uri' => 'string,optional',
         'base64' => 'string,optional',
			'name' => 'string,optional',
			'comment' => 'string,optional',
			'help' => 'bool,optional'
		);
	}
	if (!isset ($_SESSION['glpiID'])) {
		return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_NOTAUTHENTICATED);
	}
	$ticket = new Job();

	if (!isset ($params['ticket'])) {
		return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_MISSINGPARAMETER, '', 'ticket');

	} else
		if (!is_numeric($params['ticket'])) {
			return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_BADPARAMETER, '', 'ticket');

		} else
			if (!$ticket->getFromDB($params['ticket']) || !$ticket->canView()) {
				return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_NOTFOUND);
			}
	if (!$ticket->canAddFollowups()) {
		return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_NOTALLOWED);
	}

   if (!isset ($params['name ']) && !empty ($params['name'])) {
      $input['name'] = addslashes($params['name']);
   } else {
      $input['name'] = addslashes($LANG['tracking'][24] . " " . $ticket->fields['ID']);
   }

	if (!isset($params['uri']) && !isset($params['base64'])) {
		return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_MISSINGPARAMETER, '', 'uri or base64');
	}

   $filename = tempnam(GLPI_DOC_DIR . '/_tmp', 'PWS');
   $_FILES['filename']['tmp_name'] = $filename;

   if (isset($params['uri'])) {
      $content = @ file_get_contents($params['uri']);
      if (!$content) {
         return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_NOTFOUND, '', $params['uri']);
      }
      $_FILES['filename']['name'] = basename($params['uri']);

   } else if (isset($params['base64'])) {
      $content = base64_decode($params['base64']);
      if (!$content) {
         return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_BADPARAMETER, '', $params['base64']);
      }
      $_FILES['filename']['name'] = basename($input['name']);
   }

	$size = @ file_put_contents($filename, $content);
	if (!$size) {
		return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_FAILED, '', $filename);
	}
   $_FILES['filename']['size'] = $size;

	if (!isset ($params['comment ']) && !empty ($params['comment'])) {
		$input['comments'] = addslashes($params['comment']);
	}
	$input['FK_entities'] = $ticket->fields['FK_entities'];
	$input['rubrique'] = $CFG_GLPI["default_rubdoc_tracking"];
	$input['_only_if_upload_succeed'] = 1;

	if (function_exists('finfo_open') && $finfo = finfo_open(FILEINFO_MIME)) {
		$_FILES['filename']['type'] = finfo_file($finfo, $filename);
		finfo_close($finfo);
	} else
		if (function_exists('mime_content_type')) {
			$_FILES['filename']['type'] = mime_content_type($filename);
		}

	$doc = new Document();
	if ($docid = $doc->add($input)) {
		addDeviceDocument($docid, TRACKING_TYPE, $ticket->fields['ID']);
		if ($CFG_GLPI["followup_on_update_ticket"]) {
			$_FILES = false;
			$input2 = array (
				'ID' => $ticket->fields['ID'],
				'date_mod' => $_SESSION["glpi_currenttime"],
				'_doc_added' => stripslashes($doc->fields["name"])
			);
			$ticket->update($input2, true);
		}
		return plugin_webservices_method_getTicket(array (
			'ticket' => $params['ticket']), $protocol);
	}
   return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_FAILED, '',
      (isset ($_SESSION["MESSAGE_AFTER_REDIRECT"]) ? html_clean($_SESSION["MESSAGE_AFTER_REDIRECT"]) : ''));
}

/**
 * Get a ticket information, with its followup
 * for a authenticated user
 *
 * @param $params array of options (ticket, id2name)
 * @return array of hashtable
 */
function plugin_webservices_method_getTicket($params, $protocol) {
	global $DB;

	if (isset ($params['help'])) {
		return array (
			'ticket' => 'integer,mandatory',
			'id2name' => 'bool,optional',
			'help' => 'bool,optional'
		);
	}

	if (!isset ($_SESSION['glpiID'])) {
		return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_NOTAUTHENTICATED);
	}

	$ticket = new Job();

	if (!isset ($params['ticket'])) {
		return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_MISSINGPARAMETER, '', 'ticket');

	} else
		if (!is_numeric($params['ticket'])) {
			return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_BADPARAMETER, '', 'ticket=' . $params['ticket']);

		} else
			if (!$ticket->getFromDB($params['ticket']) || !$ticket->canView()) {
				return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_NOTFOUND);
			}

	$resp = array (
		'id' => $ticket->fields['ID'],
		'entities_id' => $ticket->fields['FK_entities'],
		'name' => $ticket->fields['name'],
		'date' => $ticket->fields['date'],
		'closedate' => $ticket->fields['closedate'],
		'date_mod' => $ticket->fields['date_mod'],
		'status' => $ticket->fields['status'],
		'users_id' => $ticket->fields['author'],
		'users_id_recipient' => $ticket->fields['recipient'],
		'groups_id' => $ticket->fields['FK_group'],
      'requesttypes_id' => $ticket->fields['request_type'],
		'users_id_assign' => $ticket->fields['assign'],
		'suppliers_id_assign' => $ticket->fields['assign_ent'],
		'groups_id_assign' => $ticket->fields['assign_group'],
		'itemtype' => $ticket->fields['device_type'],
		'items_id' => $ticket->fields['computer'],
		'content' => $ticket->fields['contents'],
		'urgence' => $ticket->fields['priority'],
		'impact' => $ticket->fields['priority'],
		'priority' => $ticket->fields['priority'],
		'user_email' => $ticket->fields['uemail'],
		'use_email_notification' => $ticket->fields['emailupdates'],
		'realtime' => $ticket->fields['ID'],
		'ticketscategories_id' => $ticket->fields['category'],
		'cost_time' => $ticket->fields['cost_time'],
		'cost_fixed' => $ticket->fields['cost_fixed'],
		'cost_material' => $ticket->fields['cost_material']
	);

   $ci = new CommonItem();
	if (isset ($params['id2name'])) {
      $ci->getFromDB($resp['itemtype'],$resp['items_id']);

		$resp['users_name'] = html_clean(getUserName($ticket->fields['author']));
		$resp['users_name_recipient'] = html_clean(getUserName($ticket->fields['recipient']));
		$resp['users_name_assign'] = html_clean(getUserName($ticket->fields['assign']));
		$resp['groups_name'] = html_clean(getDropdownName('glpi_groups', $ticket->fields["FK_group"]));
		$resp['groups_name_assign'] = html_clean(getDropdownName('glpi_groups', $ticket->fields["assign_group"]));
		$resp['suppliers_name_assign'] = html_clean(getDropdownName('glpi_enterprises', $ticket->fields["assign_ent"]));
		$resp['ticketscategories_name'] = html_clean(getDropdownName('glpi_dropdown_tracking_category', $ticket->fields["category"]));
         $resp['entities_name'] = html_clean(getDropdownName('glpi_entities', $resp['entities_id']));
         $resp['status_name'] = html_clean(getStatusName($resp['status']));
         $resp['items_name'] = html_clean($ci->getNameID());
         $resp['itemtype_name'] = html_clean($ci->getType());
         $resp['requesttypes_name'] = plugin_webservices_getRequestTypeName($resp['requesttypes_id']);




	}
   $resp['followups'] = array ();
   $resp['documents'] = array ();

	if (haveRight("observe_ticket", "1")) {
		$query = "SELECT * FROM `glpi_followups` WHERE `tracking` = '" . $params['ticket'] . "' ";
		if (!haveRight("show_full_ticket", "1")) {
			$query .= " AND (`private`='0' OR `author` ='" . $_SESSION["glpiID"] . "' ) ";
		}
		$query .= " ORDER BY `date` DESC";

		foreach ($DB->request($query) as $data) {
			$flwp = array (
				'id' => $data['ID'],
				'date' => $data['date'],
				'users_id' => $data['author'],
				'content' => $data['contents']
			);
			if (isset ($params['id2name'])) {
				$flwp['users_name'] = html_clean(getUserName($data['author']));
			}
			$resp['followups'][] = $flwp;
		}
      $query = "SELECT *
                FROM `glpi_docs`
                WHERE `ID` IN (SELECT `FK_doc`
                               FROM `glpi_doc_device`
                               WHERE `device_type` = ".TRACKING_TYPE."
                                 AND `FK_device` = '".$params['ticket']."')";
      foreach ($DB->request($query) as $data) {
         $doc = array (
            'id'       => $data['ID'],
            'name'     => $data['name'],
            'filename' => $data['filename'],
            'filepath' => $data['filename'],
            'sha1sum'  => sha1_file(GLPI_DOC_DIR."/".$data['filename']),
            'mime'     => $data['mime'],
            'date_mod' => $data['date_mod'],
            'comment'  => $data['comments'],
            'users_id' => $data['FK_users']
         );
         if (isset ($params['id2name'])) {
            $doc['users_name'] = html_clean(getUserName($data['FK_users']));
         }
         $resp['documents'][] = $doc;
      }
	}
	return $resp;
}

/**
 * Get a Document the authenticated user can view
 *
 * @param $params array of options (document, ticket)
 * @return a hashtable
 */
function plugin_webservices_method_getDocument($params, $protocol) {
   global $DB, $CFG_GLPI;

   if (isset ($params['help'])) {
      return array (
         'document' => 'integer,mandatory',
         'ticket' => 'interger,optional',
         'id2name' => 'bool,optional',
         'help' => 'bool,optional'
      );
   }

   $doc = new Document();

   // Authenticated ?
   if (!isset ($_SESSION['glpiID'])) {
      return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_NOTAUTHENTICATED);
   }
   // Option parameter ticket
   if (isset ($params['ticket']) && !is_numeric($params['ticket'])) {
      return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_BADPARAMETER, '',
                                      'ticket=' . $params['ticket']);
   }
   // Mandatory parameter document
   if (!isset ($params['document'])) {
      return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_MISSINGPARAMETER, '', 'document');

   } else if (!is_numeric($params['document'])) {
      return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_BADPARAMETER, '',
                                      'document=' . $params['document']);

   } else if (!$doc->getFromDB($params['document'])) {
      return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_NOTFOUND);
   }

   // Check right (from document.send.php)
   if (isset($_SESSION["glpiactiveprofile"]["interface"])
       && $_SESSION["glpiactiveprofile"]["interface"]=="central") {
      // My doc Check and Common doc right access
      if ($doc->can($params['document'],'r') || $doc->fields["FK_users"]==$_SESSION["glpiID"]) {
         $send=true;
      }

      // Knowbase Case
      if (!$send && haveRight("knowbase","r")){
         $query = "SELECT *
            FROM glpi_doc_device
            WHERE glpi_doc_device.device_type = '".KNOWBASE_TYPE."'
               AND glpi_doc_device.FK_doc='".$doc->fields["ID"]."' ";

         $result=$DB->query($query);
         if ($DB->numrows($result)>0){
            $send=true;
         }
      }
      if (!$send && haveRight("faq","r")){
         $query = "SELECT *
            FROM glpi_doc_device
               LEFT JOIN glpi_kbitems ON (glpi_kbitems.ID = glpi_doc_device.Fk_device)
            WHERE glpi_doc_device.device_type = '".KNOWBASE_TYPE."'
               AND glpi_doc_device.FK_doc='".$doc->fields["ID"]."'
               AND glpi_kbitems.faq='1'";

         $result=$DB->query($query);
         if ($DB->numrows($result)>0) {
            $send=true;
         }
      }

      // Tracking Case
      if (!$send && isset($params['ticket'])) {
         $job=new Job;

         if ($job->getFromDB($params['ticket']) && $job->can($params['ticket'],'r')) {
            $query = "SELECT *
               FROM glpi_doc_device
               WHERE glpi_doc_device.FK_device = '".$params['ticket']."'
                  AND glpi_doc_device.device_type = '".TRACKING_TYPE."'
                  AND FK_doc='".$doc->fields["ID"]."'";
            $result=$DB->query($query);
            if ($DB->numrows($result)>0)
               $send=true;
         }
      }
   } else {
      // Check if it is my doc
      if (isset($_SESSION["glpiID"]) && $doc->fields["FK_users"]==$_SESSION["glpiID"]){
         $send=true;
      } else {
         if (haveRight("faq","r") || $CFG_GLPI["public_faq"]){
            // Check if it is a FAQ document
            $query = "SELECT *
               FROM glpi_doc_device
                  LEFT JOIN glpi_kbitems ON (glpi_kbitems.ID = glpi_doc_device.Fk_device)
               WHERE glpi_doc_device.device_type = '".KNOWBASE_TYPE."'
                  AND glpi_doc_device.FK_doc='".$doc->fields["ID"]."'
                  AND glpi_kbitems.faq='1'";

            $result=$DB->query($query);
            if ($DB->numrows($result)>0) {
               $send=true;
            }
         }

         // Tracking Case
         if (!$send && isset($params['ticket'])){
            $job=new Job;

            if ($job->getFromDB($params['ticket'])
                && $job->fields["author"]==$_SESSION["glpiID"]){
               $query = "SELECT *
                  FROM glpi_doc_device
                  WHERE glpi_doc_device.FK_device = '".$params['ticket']."'
                     AND glpi_doc_device.device_type = '".TRACKING_TYPE."'
                     AND FK_doc='".$doc->fields["ID"]."'";
               $result=$DB->query($query);
               if ($DB->numrows($result)>0)
                  $send=true;
            }
         }
      }
   }
   if (!$send) {
      return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_NOTALLOWED);
   }
   $resp = array(
      'id' => $doc->fields['ID'],
      'entities_id' => $doc->fields['FK_entities'],
      'is_recursive' => $doc->fields['recursive'],
      'name' => $doc->fields['name'],
      'filename' => $doc->fields['filename'],
      'filepath' => $doc->fields['filename'],
      'documentscategories_id' => $doc->fields['rubrique'],
      'mime' => $doc->fields['mime'],
      'date_mod' => $doc->fields['date_mod'],
      'comment' => $doc->fields['comments'],
      'is_deleted' => $doc->fields['deleted'],
      'link' => $doc->fields['link'],
      'notepad' => $doc->fields['notes'],
      'users_id' => $doc->fields['FK_users'],
      'tickets_id' => $doc->fields['FK_tracking'],
      'sha1sum' => sha1_file(GLPI_DOC_DIR."/".$doc->fields['filename']),
      'base64' => base64_encode(file_get_contents(GLPI_DOC_DIR."/".$doc->fields['filename']))
      );
   if (isset ($params['id2name'])) {
      $resp['users_name'] = html_clean(getUserName($doc->fields['FK_users']));
      $resp['documentscategories_name'] = html_clean(getDropdownName('glpi_dropdown_rubdocs',
                                                                     $doc->fields['rubrique']));
   }
   return $resp;
}

/**
 * List the tickets  for a authenticated user
 *
 * @param $params array of options (author, group, category, status, startdate, enddate, itemtype)
 * @return array of hashtable
 */
function plugin_webservices_method_listTickets($params, $protocol) {
	global $DB, $CFG_GLPI;

	if (isset ($params['help'])) {
		return array (
			'count' => 'bool,optional',
			'start' => 'integer,optional',
			'limit' => 'integer,optional',
			'user' => 'integer,optional',
         'recipient' => 'integer,optional',
         'mine' => 'bool,optional',
			'group' => 'integer,optional',
         'mygroups' => 'bool,optional',
			'category' => 'integer,optional',
			'status' => 'string,optional',
			'startdate' => 'datetime,optional',
			'enddate' => 'datetime,optional',
			'itemtype' => 'integer,optional',
			'item' => 'integer,optional',
         'entity' => 'integer,optional',
         'id2name' => 'bool,optional',
         'order'   => 'array,optional',
			'help' => 'bool,optional'
		);
	}

	if (!isset ($_SESSION['glpiID'])) {
		return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_NOTAUTHENTICATED);
	}

	$resp = array ();
	$start = 0;
	if (isset ($params['start']) && is_numeric($params['start'])) {
		$start = $params['start'];
	}
	$limit = $CFG_GLPI["list_limit_max"];
	if (isset ($params['limit']) && is_numeric($params['limit'])) {
		$limit = $params['limit'];
	}

   // User (victim)
   if (isset ($params['user'])) {
      if (!is_numeric($params['user']) || $params['user'] < 0) {
         return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_BADPARAMETER, '', 'user');
      }
      if (haveRight('show_all_ticket', '1') || haveRight('show_group_ticket', '1')
          || $params['user'] == $_SESSION['glpiID']) {
         // restrict to author parameter
         $where = " AND `author`='" . $params['user'] . "'";
      } else {
         return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_NOTALLOWED);
      }
   } else {
      if (haveRight('show_all_ticket', '1') || haveRight('show_group_ticket', '1')) {
         $where = ''; // Restrict will come from group (if needed)
      } else {
         // Only connected user's tickets'
         $where = " AND `author`='" . $_SESSION['glpiID'] . "'";
      }
   }


	// Group
	if (isset ($params['group'])) {

		if (!is_numeric($params['group']) || $params['group'] < 0) {
			return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_BADPARAMETER, '', 'group');
		}
		if (haveRight('show_all_ticket', '1') || (haveRight('show_group_ticket', '1') && in_array($params['group'], $_SESSION['glpigroups']))) {
			// restrict to group parameter
			$where = " AND `FK_group`='" . $params['group'] . "'";
		} else {
			return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_NOTALLOWED);
		}
	} else
		if (haveRight('show_group_ticket', '1') && !haveRight('show_all_ticket', '1')) {
			// Connected user's group'
         if (count($_SESSION['glpigroups'])>0) {
            $where = " AND `FK_group` IN (" . implode(',', $_SESSION['glpigroups']) . ")";
         } else {
            $where = " AND `author`='" . $_SESSION['glpiID'] . "'";
         }
		}

	// Security
	if (empty ($where) && !haveRight('show_all_ticket', '1')) {
		return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_NOTALLOWED, '', 'profil');
	}


   // Recipient (person creating the ticket)
   if (isset ($params['recipient'])) {
      if (!is_numeric($params['recipient']) || $params['recipient'] < 0) {
         return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_BADPARAMETER, '', 'recipient');
      }
      // restrict to recipient parameter
      $where = " AND `recipient`='" . $params['recipient'] . "'";
   }

   // Mine (user or recipient for the ticket)
   if (isset ($params['mine'])) {
      $where = " AND (`author`='" . $_SESSION['glpiID'] . "'
                      OR `recipient`='" . $_SESSION['glpiID'] . "')";
   }

   // Mygroups
   if (isset ($param['mygroups'])) {
      $where = " AND `FK_group` IN (" . implode(',', $_SESSION['glpigroups']) . ")";
   }

   // Entity
   if (isset ($params['entity'])) {
      if (!haveAccessToEntity($params['entity'])) {
         return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_NOTALLOWED, '', 'entity');
      }
      $where = getEntitiesRestrictRequest("WHERE", "glpi_tracking", '', $params['entity']) . $where;
   } else {
      $where = getEntitiesRestrictRequest("WHERE", "glpi_tracking") . $where;
   }

	// Category
	if (isset ($params['category'])) {
		if (!is_numeric($params['category']) || $params['category'] <= 0) {
			return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_BADPARAMETER, '', 'category');
		}
		$where .= " AND " . getRealQueryForTreeItem("glpi_dropdown_tracking_category", $params['category'], "glpi_tracking.category");
	}

	// Status
	if (isset ($params['status'])) {
		if (!in_array($params['status'], array (
				'notold',
				'old',
				'process',
				'waiting',
				'old_done',
				'new',
				'old_notdone',
				'assign',
				'plan',
            'all'))) {
			return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_BADPARAMETER, '', 'status');
		}
		switch ($params['status']) {
         case 'all':
            // No restriction
            break;
			case "notold" :
				$where .= " AND `glpi_tracking`.`status` IN ('new','plan','assign','waiting')";
				break;
			case "old" :
				$where .= " AND `glpi_tracking`.`status` IN ('old_done', 'old_notdone')";
				break;
			case "process" :
				$where .= " AND `glpi_tracking`.`status` IN ('plan', 'assign' )";
				break;
			default :
				$where .= " AND `glpi_tracking`.`status` = '" . $params['status'] . "' ";
				break;
		}
	}

	// Dates
	if (isset ($params["startdate"])) {
		if (preg_match(WEBSERVICES_REGEX_DATETIME, $params["startdate"]) || preg_match(WEBSERVICES_REGEX_DATE, $params["startdate"])) {
			$where .= " AND (`glpi_tracking`.`closedate` IS NULL
			                            OR `glpi_tracking`.`closedate`>='" . $params['startdate'] . "' )";
		} else {
			return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_BADPARAMETER, '', 'date');
		}
	}
	if (isset ($params["enddate"])) {
		if (preg_match(WEBSERVICES_REGEX_DATETIME, $params["enddate"]) || preg_match(WEBSERVICES_REGEX_DATE, $params["enddate"])) {
			$where .= " AND `glpi_tracking`.`date`<='" . $params['enddate'] . "'";
		} else {
			return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_BADPARAMETER, '', 'date');
		}
	}

	if (isset ($params['itemtype'])) {
		if (!is_numeric($params['itemtype']) || $params['itemtype'] <= 0) {
			return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_BADPARAMETER, '', 'itemtype');
		}
		$where .= " AND `glpi_tracking`.`device_type`='" . $params['itemtype'] . "'";
	}
	if (isset ($params['item'])) {
		if (!isset ($params['itemtype'])) {
			return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_MISSINGPARAMETER, '', 'itemtype');
		}
		if (!is_numeric($params['item']) || $params['item'] <= 0) {
			return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_BADPARAMETER, '', 'item');
		}
		$where .= " AND `glpi_tracking`.`computer`='" . $params['item'] . "'";
	}

   $orders=array();
   if (isset ($params['order'])) {
      if (is_array($params['order'])) {
         $tab = $params['order'];
      } else {
         $tab = array($params['order']=>'DESC');
      }
      foreach ($tab as $key => $val) {
         if ($val != 'ASC') {
            $val = 'DESC';
         }
         if (in_array($key, array('id', 'date', 'closedate', 'date_mod', 'status', 'users_id',
                                  'groups_id', 'entities_id', 'priority'))) {
            $orders[] ="`$key` $val";
         } else {
            return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_BADPARAMETER, '', 'order=$key');
         }
      }
   }
   if (count($orders)) {
      $order = implode(',',$orders);
   } else {
      $order = "`date_mod` DESC";
   }
   $resp = array ();
   if (isset ($params['count'])) {
      $query = "SELECT COUNT(DISTINCT `ID`) as count FROM `glpi_tracking` " . $where;

      foreach ($DB->request($query) as $data) {
         $resp = $data;
      }
   } else {
   	$query = "SELECT `glpi_tracking`.`ID` as id,
	                    `glpi_tracking`.`FK_entities` as `entities_id`,
	                    `glpi_tracking`.`name`,
	                    `glpi_tracking`.`date`, `glpi_tracking`.`closedate`, `glpi_tracking`.`date_mod`,
	                    `glpi_tracking`.`status`,
	                    `glpi_tracking`.`author` as users_id,
	                    `glpi_tracking`.`recipient` as users_id_recipient,
	                    `glpi_tracking`.`FK_group` as groups_id,
	                    `glpi_tracking`.`priority` as urgence,
	                    `glpi_tracking`.`priority` as impact,
	                    `glpi_tracking`.`priority`,
	                    `glpi_tracking`.`category` as ticketscategories_id,
	                    `glpi_tracking`.`device_type` as itemtype, `glpi_tracking`.`computer` as items_id,
                       `glpi_tracking`.`request_type` AS requesttypes_id
	             FROM `glpi_tracking`
	             $where
	             ORDER BY $order LIMIT $start,$limit";

      $ci=new CommonItem();

      foreach ($DB->request($query) as $data) {
         if (isset ($params['id2name'])) {
            $ci->getFromDB($data['itemtype'], $data['items_id']);

            $data['users_name'] = html_clean(getUserName($data['users_id']));
            $data['users_name_recipient'] = html_clean(getUserName($data['users_id_recipient']));
            $data['entities_name'] = html_clean(getDropdownName('glpi_entities', $data['entities_id']));
            $data['status_name'] = html_clean(getStatusName($data['status']));
            $data['items_name'] = html_clean($ci->getNameID());
            $data['itemtype_name'] = html_clean($ci->getType());
            $data['groups_name'] = html_clean(getDropdownName('glpi_groups', $data['groups_id']));
            $data['ticketscategories_name'] = html_clean(getDropdownName('glpi_dropdown_tracking_category', $data['ticketscategories_id']));
            $data['requesttypes_name'] = plugin_webservices_getRequestTypeName($data['requesttypes_id']);

         }
   		$resp[] = $data;
   	}
   }

	return $resp;
}

function plugin_webservices_method_listComputers($params, $protocol) {
	global $DB, $CFG_GLPI;

	if (isset ($params['help'])) {
		return array (
			'count' => 'bool,optional',
			'start' => 'integer,optional',
			'limit' => 'integer,optional',
			'name' => 'string,optional',
			'serial' => 'string,optional',
			'otherserial' => 'string,optional',
			'help' => 'bool,optional'
		);
	}

	if (!isset ($_SESSION['glpiID'])) {
		return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_NOTAUTHENTICATED);
	}

	$where = getEntitiesRestrictRequest(" WHERE ", "glpi_computers");

	if (isset ($params['name'])) {
		$where .= " AND glpi_computers.name LIKE '%" . $params['name'] . "%'";
	}
	if (isset ($params['serial'])) {
		$where .= " AND glpi_computers.serial LIKE '%" . $params['serial'] . "%'";
	}
	if (isset ($params['otherserial'])) {
		$where .= " AND glpi_computers.otherserial LIKE '%" . $params['otherserial'] . "%'";
	}

	$resp = array ();
	if (isset ($params['count'])) {
		$query = "SELECT COUNT(DISTINCT glpi_computers.ID) as count FROM glpi_computers " . $where;

      foreach ($DB->request($query) as $data) {
         $resp = $data;
      }
	} else {

		$start = 0;
		$limit = $CFG_GLPI["list_limit_max"];
		if (isset ($params['limit']) && is_numeric($params['limit'])) {
			$limit = $params['limit'];
		}
		if (isset ($params['start']) && is_numeric($params['start'])) {
			$start = $params['start'];
		}

		$query = "SELECT DISTINCT `glpi_computers`.`ID` AS `id`, `glpi_computers`.`name`, `glpi_computers`.`serial`, `glpi_computers`.`otherserial`,
		             `glpi_computers`.`FK_entities` as `entities_id`, `glpi_entities`.`completename` AS entities_name,
		             `glpi_dropdown_locations`.`completename` AS `locations_name`
		         FROM `glpi_computers`
		         LEFT JOIN `glpi_entities` ON (`glpi_computers`.`FK_entities`=`glpi_entities`.`ID`)
		         LEFT JOIN `glpi_dropdown_locations` ON (`glpi_computers`.`location`=`glpi_dropdown_locations`.`ID`)
		         " . $where . "
		         ORDER BY glpi_computers.ID LIMIT $start,$limit";

   	foreach ($DB->request($query) as $data) {
   		$resp[] = $data;
   	}
   }
	return $resp;

}
/**
 * Get a Computer
 * for a authenticated user
 *
 * @param $params array of options (computer, id2name)
 * @return hashtable -fields of glpi_computer
 */
function plugin_webservices_method_getComputer($params, $protocol) {
	global $DB;

	if (isset ($params['help'])) {
		return array (
			'computer' => 'integer',
			'id2name' => 'bool,optional',
			'infocoms' => 'bool,optional',
         'contracts' => 'bool,optional',
			'help' => 'bool,optional'
		);
	}
	if (!isset ($_SESSION['glpiID'])) {
		return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_NOTAUTHENTICATED);
	}

	$computer = new Computer();

	if (!isset ($params['computer'])) {
		return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_MISSINGPARAMETER, '', 'computer');

	} else
		if (!is_numeric($params['computer'])) {
			return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_BADPARAMETER, '', 'computer=' . $params['computer']);

		} else
			if (!$computer->getFromDB($params['computer']) || !$computer->can($params['computer'], 'r')) {
				return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_NOTFOUND);
			}
	// TODO : more dropdown value
	// TODO : fields $params (list of fields to retrive)

	$resp = array (
		'id' => $computer->fields["ID"],
		'entities_id' => $computer->fields["FK_entities"],
		'name' => $computer->fields["name"],
		'serial' => $computer->fields["serial"],
		'otherserial' => $computer->fields["otherserial"],
		'contact' => $computer->fields["contact"],
		'contactnum' => $computer->fields["contact_num"],
		'users_id_tech' => $computer->fields["tech_num"],
		'comment' => $computer->fields["comments"],
		'date_mod' => $computer->fields["date_mod"],
		'operatingsystems_id' => $computer->fields["os"],
		'operatingsystemsversions_id' => $computer->fields["os_version"],
		'operatingsystemsservicepacks_id' => $computer->fields["os_sp"],
		'os_license_number' => $computer->fields["os_license_number"],
		'os_licenseid' => $computer->fields["os_license_id"],
		'autoupdatesystems_id' => $computer->fields["auto_update"],
		'domains_id' => $computer->fields["domain"],
		'networks_id' => $computer->fields["network"],
		'is_template' => $computer->fields["is_template"],
		'template_name' => $computer->fields["tplname"],
		'manufacturers_id' => $computer->fields["FK_glpi_enterprise"],
		'is_deleted' => $computer->fields["deleted"],
		'notepad' => $computer->fields["notes"],
		'is_ocs_import' => $computer->fields["ocs_import"],
		'user_id' => $computer->fields["FK_users"],
		'groups_id' => $computer->fields["FK_groups"],
		'states_id' => $computer->fields["state"],
		'ticket_tco' => $computer->fields["ticket_tco"],
      'states_id' => $computer->fields["state"]
	);
	if (isset ($params['id2name'])) {
		$resp['autoupdatesystems_name'] = html_clean(getDropdownName('glpi_dropdown_auto_update', $computer->fields["auto_update"]));
		$resp['networks_name'] = html_clean(getDropdownName('glpi_dropdown_network', $computer->fields["network"]));
		$resp['domains_name'] = html_clean(getDropdownName('glpi_dropdown_domain', $computer->fields["domain"]));
		$resp['manufacturers_name'] = html_clean(getDropdownName('glpi_dropdown_manufacturer', $computer->fields["FK_glpi_enterprise"]));
      $resp['states_name'] = html_clean(getDropdownName('glpi_dropdown_state', $computer->fields["state"]));
	}

	if (isset ($params['infocoms'])) {
		$infocoms = plugin_webservices_method_getComputerInfoComs($params, $protocol);
		if (!plugin_webservice_isError($protocol, $infocoms)) {
			$resp['infocoms'] = $infocoms;
		}
	}

   if (isset ($params['contracts'])) {
      $infocoms = plugin_webservices_method_getComputerContracts($params, $protocol);
      if (!plugin_webservice_isError($protocol, $infocoms)) {
         $resp['contracts'] = $infocoms;
      }
   }

	return $resp;
}

/**
 * Get Infocom for a Computer
 * for a authenticated user
 *
 * @param $params array of options (computer)
 * @return hashtable -fields of glpi_computer
 */
function plugin_webservices_method_getComputerInfoComs($params, $protocol) {
	global $DB;

   if (isset ($params['help'])) {
      return array (
         'computer' => 'integer',
         'id2name' => 'bool,optional',
         'help' => 'bool,optional'
      );
   }

	if (!isset ($_SESSION['glpiID'])) {
		return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_NOTAUTHENTICATED);
	}

	if (!isset ($params['computer'])) {
		return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_MISSINGPARAMETER, '', 'computer');

	} else
		if (!is_numeric($params['computer'])) {
			return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_BADPARAMETER, '', 'computer=' . $params['computer']);
		}

	return plugin_webservices_getItemInfoComs($protocol, COMPUTER_TYPE, $params['computer'], isset ($params['id2name']));
}


/**
 * Get contracts for a Computer
 * for a authenticated user
 *
 * @param $params array of options (computer)
 * @return hashtable -fields of glpi_contracts
 */
function plugin_webservices_method_getComputerContracts($params, $protocol) {
   global $DB;

   if (isset ($params['help'])) {
      return array (
         'computer' => 'integer',
         'id2name' => 'bool,optional',
         'help' => 'bool,optional'
      );
   }

   if (!isset ($_SESSION['glpiID'])) {
      return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_NOTAUTHENTICATED);
   }

   if (!isset ($params['computer'])) {
      return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_MISSINGPARAMETER, '', 'computer');

   } else
      if (!is_numeric($params['computer'])) {
         return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_BADPARAMETER, '', 'computer=' . $params['computer']);
      }

   return plugin_webservices_getItemContracts($protocol, COMPUTER_TYPE, $params['computer'], isset ($params['id2name']));
}

/**
 * Return Infocom for an object
 *
 * @param $item_type : type of the item
 * @param $item_id : ID of the item
 * @param $id2name : translate id of dropdown to name
 *
 * @return a hasdtable, fields of glpi_infocoms
 */
function plugin_webservices_getItemInfoComs($protocol, $item_type, $item_id, $id2name = false) {
	global $DB;

	$infocom = new InfoCom();
	$ci = new CommonItem();

	$ci->setType($item_type, true);

	if (!$infocom->getFromDBforDevice($item_type, $item_id) || !$ci->obj->can($item_id, 'r')) {
		return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_NOTFOUND);
	}

	// TODO : more dropdown value
	// TODO : fields $params (list of fields to retrive)

	$resp = array (
		'id' => $infocom->fields["ID"],
		'items_id' => $infocom->fields["FK_device"],
		'itemtype' => $infocom->fields["device_type"],
		'buy_date' => $infocom->fields["buy_date"],
		'warranty_duration' => $infocom->fields["warranty_duration"],
		'warranty_info' => $infocom->fields["warranty_info"],
		'warranty_expiration' => getWarrantyExpir($infocom->fields["buy_date"], $infocom->fields["warranty_duration"]),
		'use_date' => $infocom->fields["use_date"],
		'suppliers_id' => $infocom->fields["FK_enterprise"],
		'order_number' => $infocom->fields["num_commande"],
		'delivry_number' => $infocom->fields["bon_livraison"],
		'immo_number' => $infocom->fields["num_immo"],
		'value' => $infocom->fields["value"],
		'warranty_value' => $infocom->fields["warranty_value"],
		'sink_time' => $infocom->fields["amort_time"],
		'sink_type' => $infocom->fields["amort_type"],
		'sink_coeff' => $infocom->fields["amort_coeff"],
		'comment' => $infocom->fields["comments"],
		'bill' => $infocom->fields["facture"],
		'budgets_id' => $infocom->fields["budget"],
		'alert' => $infocom->fields["alert"]
	);
	if ($id2name) {
		$resp['suppliers_name'] = html_clean(getDropdownName('glpi_enterprises', $infocom->fields["FK_enterprise"]));
		$resp['budgets_names'] = html_clean(getDropdownName('glpi_dropdown_budget', $infocom->fields["budget"]));
	}
	return $resp;
}

/**
 * Return Infocom for an object
 *
 * @param $item_type : type of the item
 * @param $item_id : ID of the item
 * @param $id2name : translate id of dropdown to name
 *
 * @return a hasdtable, fields of glpi_infocoms
 */
function plugin_webservices_getItemContracts($protocol, $item_type, $item_id, $id2name = false) {
   global $DB;

   $commonitem = new CommonItem;

   if (!$commonitem->getFromDB($item_type,$item_id) || !haveRight('contract','r')) {
      return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_NOTFOUND);
   }

   $contract = new Contract();

   $query = "SELECT glpi_contracts.*
      FROM glpi_contract_device, glpi_contracts
      LEFT JOIN glpi_entities ON (glpi_contracts.FK_entities=glpi_entities.ID)
      WHERE glpi_contracts.ID=glpi_contract_device.FK_contract AND glpi_contract_device.FK_device = '$item_id'
         AND glpi_contract_device.device_type = '$item_type'
      ".getEntitiesRestrictRequest(" AND","glpi_contracts")."
      ORDER BY glpi_contracts.name";
   logInFile(LOGFILENAME,$query);
   $result = $DB->query($query);

   while ($datas = $DB->fetch_array($result)) {
   	$contract->getFromDB($datas['ID']);
      $resp[$datas['ID']] = array (
      'id' => $contract->fields["ID"],
      'name' => $contract->fields["name"],
      'num' => $contract->fields["num"],
      'cost' => $contract->fields["cost"],
      'contractstypes_id' => $contract->fields["contract_type"],
      'begin_date' => ($contract->fields["begin_date"]!=null?$contract->fields["begin_date"]:''),
      'duration' => $contract->fields["duration"],
      'notice' => $contract->fields["notice"],
      'periodicity' => $contract->fields["periodicity"],
      'billing' => $contract->fields["facturation"],
      'comment' => $contract->fields["comments"],
      'accounting_number' => ($contract->fields["compta_num"]!=null?$contract->fields["compta_num"]:''),
      'week_begin_hour' => $contract->fields["week_begin_hour"],
      'week_end_hour' => $contract->fields["week_end_hour"],
      'saturday_begin_hour' => $contract->fields["saturday_begin_hour"],
      'saturday_end_hour' => $contract->fields["saturday_end_hour"],
      'monday_begin_hour' => $contract->fields["monday_begin_hour"],
      'monday_end_hour' => $contract->fields["monday_end_hour"],
      'use_monday' => $contract->fields["monday"],
      'use_saturday' => $contract->fields["saturday"],
      'max_links_allowed' => $contract->fields["device_countmax"],
      'notepad' => $contract->fields["notes"],
      'alert' => $contract->fields["alert"],
      'renewal' => $contract->fields["renewal"]
      );
      if (isset($params['id2name'])) {
      	$resp[$datas['ID']]['contractstypes_name'] = html_clean(getDropdownName('glpi_contract_type',$contract->fields["contract_type"]));
      }
   }

   return $resp;
}

/**
 * List inventory objects (global search)
 * @param params the input parameters
 * @param protocol the commonication protocol used
 */
function plugin_webservices_method_listInventoryObjects($params, $protocol) {
	global $DB, $CFG_GLPI;

	if (isset ($params['help'])) {
		return array (
			'start' => 'integer,optional',
			'limit' => 'integer,optional',
			'name' => 'string,optional',
			'serial' => 'string,optional',
			'otherserial' => 'string,optional',
			'help' => 'bool,optional'
		);
	}

	if (!isset ($_SESSION['glpiID'])) {
		return plugin_webservices_Error($protocol, WEBSERVICES_ERROR_NOTAUTHENTICATED);
	}

	$resp = array ();

	$start = 0;
	$limit = $CFG_GLPI["list_limit_max"];
	if (isset ($params['limit']) && is_numeric($params['limit'])) {
		$limit = $params['limit'];
	}
	if (isset ($params['start']) && is_numeric($params['start'])) {
		$start = $params['start'];
	}

	$query = "SELECT glpi_computers.name AS name, glpi_computers.ID AS id, glpi_entities.completename AS entities_name, glpi_entities.ID AS entities_id, glpi_computers.serial AS serial, glpi_computers.otherserial AS otherserial, 1 AS itemtype
	                  FROM glpi_computers
	                  LEFT JOIN glpi_entities ON (glpi_computers.FK_entities = glpi_entities.ID)
	                  WHERE glpi_computers.deleted='0' AND glpi_computers.is_template='0' " . getEntitiesRestrictRequest(" AND ", "glpi_computers") . plugin_webservices_listInventoryObjectsRequestParameters($params, "glpi_computers") . "
	                  UNION
	                  SELECT glpi_printers.name AS name, glpi_printers.ID AS id, glpi_entities.completename AS entities_name, glpi_entities.ID AS entities_id, glpi_printers.serial AS serial, glpi_printers.otherserial AS otherserial, 3 AS itemtype
	                  FROM glpi_printers
	                  LEFT JOIN glpi_entities ON (glpi_printers.FK_entities = glpi_entities.ID)
	                  WHERE glpi_printers.deleted='0' AND glpi_printers.is_template='0' " . getEntitiesRestrictRequest(" AND ", "glpi_printers") . plugin_webservices_listInventoryObjectsRequestParameters($params, "glpi_printers") . "
	                  UNION
	                  SELECT glpi_monitors.name AS name, glpi_monitors.ID AS id, glpi_entities.completename AS entities_name, glpi_entities.ID AS entities_id, glpi_monitors.serial AS serial, glpi_monitors.otherserial AS otherserial, 4 AS itemtype
	                  FROM glpi_monitors
	                  LEFT JOIN glpi_entities ON (glpi_monitors.FK_entities = glpi_entities.ID)
	                  WHERE glpi_monitors.deleted='0' AND glpi_monitors.is_template='0' " . getEntitiesRestrictRequest(" AND ", "glpi_monitors") . plugin_webservices_listInventoryObjectsRequestParameters($params, "glpi_monitors") . "
	                  UNION
	                  SELECT glpi_peripherals.name AS name, glpi_peripherals.ID AS id, glpi_entities.completename AS entities_name, glpi_entities.ID AS entities_id, glpi_peripherals.serial AS serial, glpi_peripherals.otherserial AS otherserial, 5 AS itemtype
	                  FROM glpi_peripherals
	                  LEFT JOIN glpi_entities ON (glpi_peripherals.FK_entities = glpi_entities.ID)
	                  WHERE glpi_peripherals.deleted='0' AND glpi_peripherals.is_template='0' " . getEntitiesRestrictRequest(" AND ", "glpi_peripherals") . " AND ( (glpi_peripherals.name LIKE '%XP%' ) ) " . plugin_webservices_listInventoryObjectsRequestParameters($params, "glpi_peripherals") . "
	                  UNION
	                  SELECT glpi_networking.name AS name, glpi_networking.ID AS id, glpi_entities.completename AS entities_name, glpi_entities.ID AS entities_id, glpi_networking.serial AS serial, glpi_networking.otherserial AS otherserial, 2 AS itemtype
	                  FROM glpi_networking
	                  LEFT JOIN glpi_entities ON (glpi_networking.FK_entities = glpi_entities.ID)
	                  WHERE glpi_networking.deleted='0' AND glpi_networking.is_template='0' " . getEntitiesRestrictRequest(" AND ", "glpi_networking") . plugin_webservices_listInventoryObjectsRequestParameters($params, "glpi_networking") . "
	                  UNION
	                  SELECT glpi_phones.name AS name, glpi_phones.ID AS id, glpi_entities.completename AS entities_name, glpi_entities.ID AS entities_id, glpi_phones.serial AS serial, glpi_phones.otherserial AS otherserial, 23 AS itemtype
	                  FROM glpi_phones
	                  LEFT JOIN glpi_entities ON (glpi_phones.FK_entities = glpi_entities.ID)
	                  WHERE glpi_phones.deleted='0' AND glpi_phones.is_template='0' " . getEntitiesRestrictRequest(" AND ", "glpi_phones") . plugin_webservices_listInventoryObjectsRequestParameters($params, "glpi_phones") . "
	                  ORDER BY name LIMIT $start,$limit";

	$commonitem = new CommonItem;

	foreach ($DB->request($query) as $data) {
		$commonitem->setType($data['itemtype']);
		$data['itemtype_name'] = html_clean($commonitem->getType());
		$resp[] = $data;
	}
	return $resp;

}

/**
 * Contruct parameters restriction for listInventoryObjects sql request
 * @param the input parameters
 */
function plugin_webservices_listInventoryObjectsRequestParameters($params, $table) {
	$where = "";
	if (isset ($params['name'])) {
		$where .= " AND $table.name LIKE '%" . $params['name'] . "%'";
	}
	if (isset ($params['serial'])) {
		$where .= " AND $table.serial LIKE '%" . $params['serial'] . "%'";
	}
	if (isset ($params['otherserial'])) {
		$where .= " AND $table.otherserial LIKE '%" . $params['otherserial'] . "%'";
	}
	return $where;
}
?>
