<?php


/*
 ----------------------------------------------------------------------
 GLPI - Gestionnaire Libre de Parc Informatique
 Copyright (C) 2003-2008 by the INDEPNET Development Team.

 http://indepnet.net/   http://glpi-project.org/
 ----------------------------------------------------------------------

 LICENSE

 This file is part of GLPI.

 GLPI is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 GLPI is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with GLPI; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ------------------------------------------------------------------------
 */

// ----------------------------------------------------------------------
// Original Author of file: Remi Collet
// Purpose of file: Methods for WebServices plugin
//    Methods which manage the user session
// ----------------------------------------------------------------------

if (!defined('GLPI_ROOT')) {
	die("Sorry. You can't access directly to this file");
}

/**
 * This method try to identicate a user
 *
 * @param $params array of options
 * => login_name : mandatory user name
 * => login_password : mandatory user password
 * => other : optionnal values for post action
 *
 * @return an response ready to be encode
 * => id of the user
 * => name of the user
 * => realname of the user
 * => firstname of the user
 * => session : ID of the session for future call
 */
function plugin_webservices_method_doLogin($params,$protocol) {

	global $DB, $LANG, $CFG_GLPI;

	if (isset ($params['help'])) {
		return array (
			'login_name' => 'string,mandatory',
			'login_password' => 'string,mandatory',
			'help' => 'bool,optional'
		);
	}

	if (!isset ($params['login_name']) || empty ($params['login_name']) || !isset ($params['login_password']) || empty ($params['login_password'])) {
		return plugin_webservices_Error($protocol,WEBSERVICES_ERROR_MISSINGPARAMETER);
	}

	$identificat = new Identification();
	$identificat->getAuthMethods();
	$identificat->user_present = 1;
	$identificat->auth_succeded = false;

	foreach ($params as $name => $value) {
		switch ($name) {
			case 'login_name' :
			case 'login_password' :
				// Store to post, for login
				$_POST[$name] = $value;
				break;
			default :
				// Store to Session, for post login action (retrieve_more_data_from_ldap, p.e.)
				$_SESSION[$name] = $value;
				break;
		}
	}

	// ===== from login.php ====

	// exists=0 -> no exist
	// exists=1 -> exist with password
	// exists=2 -> exist without password
	$exists = $identificat->userExists(addslashes($_POST['login_name']));

	// Pas en premier car sinon on ne fait pas le blankpassword
	// First try to connect via le DATABASE
	if ($exists == 1) {

		// Without UTF8 decoding
		if (!$identificat->auth_succeded) {
			$identificat->auth_succeded = $identificat->connection_db(addslashes($_POST['login_name']), $_POST['login_password']);
			if ($identificat->auth_succeded) {
				$identificat->extauth = 0;
				$identificat->user_present = $identificat->user->getFromDBbyName(addslashes($_POST['login_name']));
				$identificat->user->fields["auth_method"] = AUTH_DB_GLPI;
				$identificat->user->fields["password"] = $_POST['login_password'];
			}

		}
	}
	elseif ($exists == 2) {
		//The user is not authenticated on the GLPI DB, but we need to get informations about him
		//The determine authentication method
		$identificat->user->getFromDBbyName(addslashes($_POST['login_name']));

		//If the user has already been logged, the method_auth and id_auth are already set
		//so we test this connection first
		switch ($identificat->user->fields["auth_method"]) {
			case AUTH_EXTERNAL :
			case AUTH_LDAP :
				if (canUseLdap()) {
					error_reporting(0);
					$identificat = try_ldap_auth($identificat, $_POST['login_name'], $_POST['login_password'], $identificat->user->fields["id_auth"]);
				}
				break;
			case AUTH_MAIL :
				if (canUseImapPop()) {
					$identificat = try_mail_auth($identificat, $_POST['login_name'], $_POST['login_password'], $identificat->user->fields["id_auth"]);
				}
				break;
			case NOT_YET_AUTHENTIFIED :
				break;
		}
	}

	//If the last good auth method is not valid anymore, we test all the methods !
	//test all the ldap servers
	if (!$identificat->auth_succeded && canUseLdap()) {
		error_reporting(0);
		$identificat = try_ldap_auth($identificat, $_POST['login_name'], $_POST['login_password']);
	}

	//test all the imap/pop servers
	if (!$identificat->auth_succeded && canUseImapPop()) {
		$identificat = try_mail_auth($identificat, $_POST['login_name'], $_POST['login_password']);
	}
	// Fin des tests de connexion

	// Ok, we have gathered sufficient data, if the first return false the user
	// are not present on the DB, so we add it.
	// if not, we update it.
	if (!$DB->isSlave() && $identificat->auth_succeded) {
		error_log("5");

		// Prepare data
		$identificat->user->fields["last_login"] = $_SESSION["glpi_currenttime"];
		if ($identificat->extauth) {
			$identificat->user->fields["_extauth"] = 1;
		}
		// Need auto add user ?
		if (!$identificat->user_present && $CFG_GLPI["auto_add_users"]) {
			$input = $identificat->user->fields;
			unset ($identificat->user->fields);
			$identificat->user->add($input);
		} else
			if (!$identificat->user_present) { // Auto add not enable so auth failed
				$identificat->addToError($LANG['login'][11]);
				$identificat->auth_succeded = false;
			} else
				if ($identificat->user_present) {
					// update user and Blank PWD to clean old database for the external auth
					$identificat->user->update($identificat->user->fields);

					if ($identificat->extauth) {
						$identificat->user->blankPassword();
					}
				}
	}
	// GET THE IP OF THE CLIENT
	$ip = (getenv("HTTP_X_FORWARDED_FOR") ? getenv("HTTP_X_FORWARDED_FOR") : getenv("REMOTE_ADDR"));

	$identificat->initSession();

	if ($identificat->auth_succeded) {
		return (array (
			'id' => $_SESSION["glpiID"],
			'name' => $_SESSION["glpiname"],
			'realname' => $_SESSION["glpirealname"],
			'firstname' => $_SESSION["glpifirstname"],
			'session' => session_id()
		));
	} else {
		return plugin_webservices_Error($protocol,WEBSERVICES_ERROR_LOGINFAILED);
	}
}

/**
 * This method try to identicate a user
 *
 * @param $params array of options ignored
 * @return an response ready to be encode
 * => fields of glpi_users
 */
function plugin_webservices_method_getMyInfo($params,$protocol) {

	global $DB, $LANG, $CFG_GLPI;

	if (isset ($params['help'])) {
		return array (
			'help' => 'bool,optional'
		);
	}

	if (!isset ($_SESSION['glpiID'])) {
		return plugin_webservices_Error($protocol,WEBSERVICES_ERROR_NOTAUTHENTICATED);
	}

	$user = new User();
	if ($user->getFromDB($_SESSION['glpiID'])) {
		// 0.80 return ($user->fields);
		return array (
			'id' => $user->fields['ID'],
			'name' => $user->fields['name'],
			'realname' => $user->fields['realname'],
			'firstname' => $user->fields['firstname'],
			'email' => $user->fields['email'],
			'phone' => $user->fields['phone'],
			'phone2' => $user->fields['phone2'],
			'language' => $user->fields['language'],
			'comment' => $user->fields['comments'],
			'last_login' => $user->fields['last_login'],
			'date_mod' => $user->fields['date_mod'],
			'userstitles_id' => $user->fields['title'],
			'userscategories_id' => $user->fields['type']
		);
	} else {
		return plugin_webservices_Error($protocol,WEBSERVICES_ERROR_NOTFOUND);
	}
}

/**
 * This method try to identicate a user
 *
 * @param $params array of options ignored
 * @return an response ready to be encode
 * => Nothing
 */
function plugin_webservices_method_doLogout($params) {

	if (isset ($params['help'])) {
		return array (
			'help' => 'bool,optional'
		);
	}

	$msg = "Bye ";
	if (isset ($_SESSION['glpiID'])) {
		$msg .= (empty ($_SESSION['glpifirstname']) ? $_SESSION['name'] : $_SESSION['glpifirstname']);
	}

	$id = new Identification();
	$id->destroySession();

	return array (
		"message" => $msg
	);
}

/**
 * This method try to identicate a user
 *
 * @param $params array of options ignored
 * @return an response ready to be encode
 * => fields of glpi_users
 */
function plugin_webservices_method_listMyProfiles($params,$protocol) {

	global $DB, $LANG, $CFG_GLPI;

	if (isset ($params['help'])) {
		return array (
			'help' => 'bool,optional'
		);
	}

	if (!isset ($_SESSION['glpiID'])) {
		return plugin_webservices_Error($protocol,WEBSERVICES_ERROR_NOTAUTHENTICATED);
	}

	$resp = array ();
	foreach ($_SESSION['glpiprofiles'] as $ID => $prof) {
		$resp[] = array (
			'id' => $ID,
			'name' => $prof['name'],
			'current' => ($ID == $_SESSION['glpiactiveprofile']['ID'] ? 1 : 0)
		);
	}
	return $resp;
}

/**
 * This method return the entities list allowed
 * for a authenticated users
 *
 * @param $params array of option : ignored
 *
 * @return an response ready to be encode (ID + completename)
 */
function plugin_webservices_method_listMyEntities($params,$protocol) {

	global $DB, $LANG;

	if (isset ($params['help'])) {
		return array (
			'help' => 'bool,optional'
		);
	}

	if (!isset ($_SESSION['glpiID'])) {
		return plugin_webservices_Error($protocol,WEBSERVICES_ERROR_NOTAUTHENTICATED);
	}

	$resp = array ();

	foreach ($_SESSION['glpiactiveprofile']['entities'] as $ent) {
		if ($ent['recursive']) {
			$search = getEntitySons($ent['ID']);
		} else {
			$search = $ent['ID'];
		}
		if ($ent['ID'] == 0) {
			$resp[0] = array (
				'id' => 0,
				'name' => $LANG['entity'][2],
				'entities_id' => 0,
				'completename' => $LANG['entity'][2],
				'comment' => '',
				'level' => 0,
				'recursive' => $ent['recursive'],
				'current' => (in_array(0, $_SESSION['glpiactiveentities']) ? 1 : 0)
			);
		}
		foreach ($DB->request('glpi_entities', array (
				'ID' => $search
			)) as $data) {
			$resp[$data['ID']] = array (
				'id' => $data['ID'],
				'name' => $data['name'],
				'entities_id' => $data['parentID'],
				'completename' => $data['completename'],
				'comment' => $data['comments'],
				'level' => $data['level'],
				'recursive' => $ent['recursive'],
				'current' => (in_array($data['ID'], $_SESSION['glpiactiveentities']) ? 1 : 0)
			);
		}
	}

	return $resp;
}

/**
 * Change the current profile of a authenticated user
 *
 * @param $params array of options
 *  - profile : ID of the new profile
 * @return an response ready to be encode
 *  - ID
 *  - name of the new profile
 */
function plugin_webservices_method_setMyProfile($params,$protocol) {

	global $DB, $LANG, $CFG_GLPI;

	if (isset ($params['help'])) {
		return array (
			'profile' => 'integer,mandatory',
			'help' => 'bool,optional'
		);
	}

	if (!isset ($_SESSION['glpiID'])) {
		return plugin_webservices_Error($protocol,WEBSERVICES_ERROR_NOTAUTHENTICATED);
	}
	if (!isset ($params['profile'])) {
		return plugin_webservices_Error($protocol,WEBSERVICES_ERROR_MISSINGPARAMETER, '', 'profile');
	}

	// TODO search for profile name if not an ID.
	$ID = $params['profile'];

	if (isset ($_SESSION['glpiprofiles'][$ID]) && count($_SESSION['glpiprofiles'][$ID]['entities'])) {

		changeProfile($ID);
		$resp = array (
			'id' => $_SESSION['glpiactiveprofile']['ID'],
			'name' => $_SESSION['glpiactiveprofile']['name']
		);

	} else {
		return plugin_webservices_Error(WEBSERVICES_ERROR_BADPARAMETER, '', "profile=$ID");
	}
	return $resp;
}

/**
 * Change the current entity(ies) of a authenticated user
 *
 * @param $params array of options
 *  - entity : ID of the new entity or "all"
 *  - recursive : 1 to see children
 * @return like plugin_webservices_method_listEntities
 */
function plugin_webservices_method_setMyEntity($params) {

	global $DB, $LANG, $CFG_GLPI;

	if (isset ($params['help'])) {
		return array (
			'entity' => 'integer,mandatory',
			'help' => 'bool,optional'
		);
	}

	if (!isset ($_SESSION['glpiID'])) {
		return plugin_webservices_Error(WEBSERVICES_ERROR_NOTAUTHENTICATED);
	}
	if (!isset ($params['entity'])) {
		return plugin_webservices_Error(WEBSERVICES_ERROR_MISSINGPARAMETER, '', 'entity');
	}

	if (changeActiveEntities($params['entity'], (isset ($params['recursive']) && $params['recursive']))) {
		return plugin_webservices_method_listEntities($_SESSION['glpiactiveentities'], $params);
	}
	return plugin_webservices_Error(WEBSERVICES_ERROR_BADPARAMETER, '', "entity=" . $params['entity']);
}

/**
 * Recovery session
 * @param session the session ID
 * @destroy try to destroy current session, in order to recover the stored one
 * TODO : use it for xmlrpc.php
 */
function plugin_webservices_setSession($session) {

   $current = session_id();
   $session = trim($session);

   if (file_exists(GLPI_ROOT . "/config/config_path.php")) {
      include_once (GLPI_ROOT . "/config/config_path.php");
   }
   if (!defined("GLPI_SESSION_DIR")) {
      define("GLPI_SESSION_DIR", GLPI_ROOT . "/files/_sessions");
   }

   if ($session!=$current && !empty($current)) {
      session_destroy();
   }
   if ($session!=$current && !empty($session)) {
      session_save_path(GLPI_SESSION_DIR);
      session_id($session);
      session_start();
   }
}

/**
 * Standard method execution : checks if client can execute method + manage session
 *
 * @param $method string method name
 * @param $params array the method parameters
 * @return array the method response
 */
function plugin_webservice_execute($method,$params,$protocol) {
	global $LANG, $DB, $WEBSERVICES_METHOD;

	$iptxt = (isset ($_SERVER["HTTP_X_FORWARDED_FOR"]) ? $_SERVER["HTTP_X_FORWARDED_FOR"] : $_SERVER["REMOTE_ADDR"]);
	$ipnum = ip2long($iptxt);

	$plug = new Plugin();
	if ($plug->isActivated('webservices')) {

		if (isset ($params['session'])) {
			plugin_webservices_setSession($params['session']);
		}

		// Build query for security check
      $sql = "SELECT * FROM glpi_plugin_webservices
              WHERE '" . addslashes($method) . "' REGEXP pattern
              AND active=1
              AND (ip_start IS NULL OR (ip_start<=$ipnum AND ip_end>=$ipnum)) ";

		if (isset ($params["username"])) {
			$username = addslashes($params["username"]);
			$password = md5(isset ($params["password"]) ? $params["username"] : '');

			$sql .= " AND (username IS NULL OR (username='$username' AND password='$password')) ";

			unset ($params["username"]);
			unset ($params["password"]);
		} else {
			$username = 'anonymous';
			$sql .= " AND username IS NULL ";
		}
		$deflate = $debug = $log = false;
		$entities = array ();
		if (isset ($_SESSION['glpiID']) && isset ($_SESSION['glpiactiveentities'])) {
			// $entities = $_SESSION['glpiactiveentities'];
			$username = $_SESSION['glpiname']; // for log (no t for SQL request)
		}

		foreach ($DB->request($sql) as $data) {
			// Check matching rules

			// Store entities for not authenticated user
			if (!isset ($_SESSION['glpiID'])) {
				if ($data['recursive']) {
					foreach (getEntitySons($data['FK_entities']) as $entity) {
						$entities[$entity] = $entity;
					}
				} else {
					$entities[$data['FK_entities']] = $data['FK_entities'];
				}
			}

			// Where to log
			if ($data["do_log"] == 2) {
				// Log to file
				$log = LOGFILENAME;
			} else
				if ($data["do_log"] == 1) {
					// Log to History
					$log = $data["ID"];
				}
			$debug = $data['debug'];
			$deflate = $data['deflate'];
		}
		// Always log when connection denied
		if (!isset ($_SESSION['glpiID']) && !count($entities)) {
			$resp = plugin_webservices_Error($protocol,1, $LANG["login"][5]);

			// log to file (not macthing config to use history)
			logInFile(LOGFILENAME, $LANG["login"][5] . " ($username, $iptxt, $method, $protocol)\n");

		} else { // Allowed
			if (!isset ($_SESSION['glpiID'])) {
				// TODO : probably more data should be initialized hgre
				$_SESSION['glpiactiveentities'] = $entities;
			}
			// Log if configured
			if (is_numeric($log)) {
				$changes[0] = 0;
				$changes[1] = "";
				$changes[2] = $LANG["log"][55] . " ($username, $iptxt, $method, $protocol)";

				historyLog($log, PLUGIN_WEBSERVICES_TYPE, $changes, 0, HISTORY_LOG_SIMPLE_MESSAGE);
			} else
				if ($log && !$debug) {
					logInFile($log, $LANG["log"][55] . " ($username, $iptxt, $method)\n");
				}

			$defserver = ini_get('zlib.output_compression');
			if ($deflate && !$defserver) {
				// Globally off, try to enable for this client
				// This only work on PHP > 5.3.0
				ini_set('zlib.output_compression', 'On');
			}
			if (!$deflate && $defserver) {
				// Globally on, disable for this client
				ini_set('zlib.output_compression', 'Off');
			}

			if (!isset ($WEBSERVICES_METHOD[$method])) {
				$resp = plugin_webservices_Error($protocol,2, "Unknown method ($method)");

			} else
				if (!function_exists($WEBSERVICES_METHOD[$method])) {
					$resp = plugin_webservices_Error($protocol,3, "Unknown internal function for $method",$protocol);

				} else {
					logInFile(LOGFILENAME, "Execute function " . $WEBSERVICES_METHOD[$method]." ($protocol) ");
					$resp = call_user_func($WEBSERVICES_METHOD[$method], $params,$protocol);
				}
		} // Allowed
	} else {
		$resp = plugin_webservices_Error($protocol,4, "Server not ready",$protocol);
	} // Activated

	if ($debug) {
		logInFile(LOGFILENAME, $LANG["log"][55] . ": $username, $iptxt\n" .
		"Protocol : $protocol, Method: $method\nParams: " .
		 (count($params) ? print_r($params, true) : "none\n") .
		"Compression: Server:$defserver/" . ini_get('zlib.output_compression') .
		", Config:$deflate, Agent:" . (isset ($_SERVER['HTTP_ACCEPT_ENCODING']) ? $_SERVER['HTTP_ACCEPT_ENCODING'] : '?') . "\n" .
		"Response: " . print_r($resp, true));
	}

	return $resp;
}
?>
