<?php
/*
 * @version $Id: HEADER 7762 2009-01-06 18:30:32Z moyo $
 -------------------------------------------------------------------------
 GLPI - Gestionnaire Libre de Parc Informatique
 Copyright (C) 2003-2009 by the INDEPNET Development Team.

 http://indepnet.net/   http://glpi-project.org
 -------------------------------------------------------------------------

 LICENSE

 This file is part of GLPI.

 GLPI is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 GLPI is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with GLPI; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 --------------------------------------------------------------------------
 */

// ----------------------------------------------------------------------
// Original Author of file: Remi Collet
// Purpose of file: Plugin setup.
// ----------------------------------------------------------------------
// Init the hooks of the plugins -Needed
function plugin_init_webservices() {
	global $PLUGIN_HOOKS,$LANG,$CFG_GLPI;

	// Params : plugin name - string type - number - class - table - form page
	array_push($CFG_GLPI["specif_entities_tables"], "glpi_plugin_webservices");
		registerPluginType('webservices', 'PLUGIN_WEBSERVICES_TYPE', 5050, array(
			'classname'  => 'WebServices',
			'tablename'  => 'glpi_plugin_webservices',
			'typename'   => 'WebServices',
			'formpage'   => 'front/plugin_webservices.form.php',
			'searchpage' => 'index.php',
			'specif_entities_tables' => true,
			'recursive_type' => true
			));

	if (haveRight("config", "r")) {
		//$PLUGIN_HOOKS['config_page']['webservices'] = 'index.php';

		$PLUGIN_HOOKS['menu_entry']['webservices'] = true;
		$PLUGIN_HOOKS['submenu_entry']['webservices']['search'] = 'index.php';
		$PLUGIN_HOOKS['submenu_entry']['webservices']['add'] = 'front/plugin_webservices.form.php?new=1';

	}
	$PLUGIN_HOOKS['webservices']['webservices'] = 'plugin_webservices_registerMethods';
}

// Get the name and the version of the plugin - Needed
function plugin_version_webservices(){
	global $LANG;

	return array(
		'name'    => $LANG["webservices"][0],
		'version' => '0.4.0',
		'author' => 'Remi Collet, Nelly Lasson',
		'homepage'=> 'https://forge.indepnet.net/projects/show/webservices',
		'minGlpiVersion' => '0.72',// For compatibility / no install in version < 0.72
	);
}

// Install process for plugin : need to return true if succeeded
function plugin_webservices_install(){
	global $DB, $LANG;

	if (!TableExists("glpi_plugin_webservices")) {
		$sql =  "CREATE TABLE `glpi_plugin_webservices` (
			`ID` INT NOT NULL AUTO_INCREMENT,
			`FK_entities` INT NOT NULL DEFAULT '0',
			`recursive` TINYINT( 1 ) NOT NULL DEFAULT '0',
			`name` VARCHAR( 255 ) NOT NULL ,
			`pattern` VARCHAR( 255 ) NOT NULL ,
			`ip_start` BIGINT NULL ,
			`ip_end` BIGINT NULL ,
			`username` VARCHAR( 255 ) NULL ,
			`password` VARCHAR( 255 ) NULL ,
			`do_log` TINYINT NOT NULL DEFAULT '0',
         `debug` TINYINT NOT NULL DEFAULT '0',
			`active` TINYINT NOT NULL DEFAULT '0',
         `deflate` TINYINT NOT NULL DEFAULT '0',
			`comments` TEXT NULL ,
			PRIMARY KEY ( `ID` )
			) ENGINE = MYISAM CHARACTER SET utf8 COLLATE utf8_unicode_ci ";
		$DB->query($sql) or die ("SQL Error");

		$sql = "INSERT INTO `glpi_plugin_webservices`
			(`ID`, `FK_entities`, `recursive`, `name`, `pattern`, `ip_start`,
          `ip_end` , `do_log`, `active`, `comments`)
			VALUES (NULL, 0, 1, '".$LANG["webservices"][100]."', '.*', INET_ATON('127.0.0.1'),
                 INET_ATON('127.0.0.1'), 1, 1, '".$LANG["webservices"][101]."')";
		$DB->query($sql);
	}
   if (!FieldExists('glpi_plugin_webservices', 'deflate')) {
      $DB->query("ALTER TABLE `glpi_plugin_webservices`
                  ADD `deflate` TINYINT NOT NULL DEFAULT '0' AFTER `active`") or die ("SQL Error");
   }
   if (!FieldExists('glpi_plugin_webservices', 'debug')) {
      $DB->query("ALTER TABLE `glpi_plugin_webservices`
                  ADD `debug` TINYINT NOT NULL DEFAULT '0' AFTER `do_log`") or die ("SQL Error");
   }

	return true;
}

// Uninstall process for plugin : need to return true if succeeded
function plugin_webservices_uninstall(){
	global $DB;

	if (TableExists("glpi_plugin_webservices")) {
		$sql =  "DROP TABLE `glpi_plugin_webservices`";
		$DB->query($sql) or die ("SQL Error");
	}
	return true;
}


// Optional : check prerequisites before install : may print errors or add to message after redirect
function plugin_webservices_check_prerequisites(){
	global $LANG;

	if (GLPI_VERSION < 0.72){

		echo $LANG["webservices"][21]. "0.72";

   } else if (!extension_loaded("soap")) {

      echo $LANG["webservices"][22]. "soap";

	} else if (!function_exists("xmlrpc_encode")) {

		echo $LANG["webservices"][22]. "xmlrpc";

	} else {

		return true;

	}
}


// Uninstall process for plugin : need to return true if succeeded : may display messages or add to message after redirect
function plugin_webservices_check_config(){
	return TableExists("glpi_plugin_webservices");
}

// Define rights for the plugin types
function plugin_webservices_haveTypeRight($type,$right){
	return haveRight("config",$right);
}

?>